# ro_command.py

"""
Basic command functions for ro, research object manager
"""

import sys
import os
import os.path
import readline # enable input editing for raw_input
import re
import datetime
import logging
import rdflib
import shutil
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

import MiscLib.ScanDirectories

import ro_settings
import ro_utils
import ro_uriutils
from ro_annotation import annotationTypes, annotationPrefixes
from ro_metadata   import ro_metadata

from iaeval import ro_eval_minim

from sync.RosrsApi import RosrsApi
from sync.ResourceSync import ResourceSync

from zipfile import ZipFile

import urllib2

def getoptionvalue(val, prompt):
    if not val:
        if sys.stdin.isatty():
            val = raw_input(prompt)
        else:
            val = sys.stdin.readline()
            if val[-1] == '\n': val = val[:-1]
    return val

def ro_root_directory(cmdname, ro_config, rodir):
    """
    Find research object root directory

    Returns directory path string, or None if not found, in which
    case an error message is displayed.
    """
    #log.debug("ro_root_directory: cmdname %s, rodir %s"%(cmdname, rodir))
    #log.debug("                   ro_config %s"%(repr(ro_config)))
    ro_dir = ro_utils.ropath(ro_config, rodir)
    if not ro_dir:
        print ("%s: indicated directory not in configured research object directory tree: %s (%s)"%
               (cmdname, rodir, ro_config['robase']))
        return None
    if not os.path.isdir(ro_dir):
        print ("%s: indicated directory does not exist: %s"%
               (cmdname, rodir))
        return None
    manifestdir = None
    ro_dir_next = ro_dir
    ro_dir_prev = ""
    #log.debug("ro_dir_next %s, ro_dir_prev %s"%(ro_dir_next, ro_dir_prev))
    while ro_dir_next and ro_dir_next != ro_dir_prev:
        #log.debug("ro_dir_next %s, ro_dir_prev %s"%(ro_dir_next, ro_dir_prev))
        manifestdir = os.path.join(ro_dir_next, ro_settings.MANIFEST_DIR)
        if os.path.isdir(manifestdir):
            return ro_dir_next
        ro_dir_prev = ro_dir_next
        ro_dir_next = os.path.dirname(ro_dir_next)    # Up one directory level
    print ("%s: indicated directory is not contained in a research object: %s"%
           (cmdname, ro_dir))
    return None

# Argumemnt count checking and usage summary

def argminmax(min, max):
    return (lambda options,args: (len(args) >= min and (max == 0 or len(args) <= max)))

ro_command_usage = (
    [ ( ["help"],            argminmax(2,2),
          ["help"] )
    , ( ["config"],          argminmax(2,2),
          ["config -b <robase> -u <username> -e <useremail> -r <rosrs_uri> -t <access_token>"] )
    , ( ["create"],          argminmax(3,3),
          ["create <RO-name> [ -d <dir> ] [ -i <RO-ident> ]"] )
    , ( ["add"],             argminmax(2,3),
          ["add [ -d <dir> ] [ -a ] [ file | directory ]"] )
    , ( ["status"],          argminmax(2,2),
          ["status [ -d <dir> ]"] )
    , ( ["list","ls"],       argminmax(2,2),
          ["list [ -d <dir> ]"
          ,"ls   [ -d <dir> ]"
          ] )
    , ( ["annotate"],        (lambda options,args: (len(args) == 3 if options.graph else len(args) in [4,5])),
          ["annotate [ -d <dir> ] <file> <attribute-name> [ <attribute-value> ]"
          ,"annotate [ -d <dir> ] <file> -g <RDF-graph>"
          ] )
    , ( ["link"],        (lambda options,args: (len(args) == 3 if options.graph else len(args) in [4,5])),
          ["link [ -d <dir> ] <file> <attribute-name> [ <attribute-value> ]"
          ,"link [ -d <dir> ] <file> -g <RDF-graph>"
          ] )
    , ( ["annotations"],     argminmax(2,3),
          ["annotations [ <file> | -d <dir> ]"] )
    , ( ["push"],            argminmax(2,2),
          ["push [ -d <dir> ] [ -f ] [ -r <rosrs_uri> ] [ -t <access_token> ]"] )
    , ( ["checkout"],        argminmax(2,3),
          ["checkout [ <RO-name> [ -d <dir>] ] [ -r <rosrs_uri> ] [ -t <access_token> ]"] )
    , ( ["evaluate","eval"], argminmax(5,6),
          ["evaluate checklist [ -d <dir> ] [ -a | -l <level> ] <minim> <purpose> [ <target> ]"] )
    ])

def check_command_args(progname, options, args):
    # Check argument count for known command
    for (cmds, test, usages) in ro_command_usage:
        if args[1] in cmds:
            if not test(options, args):
                print ("%s %s: wrong number of arguments provided"%
                       (progname,args[1]))
                print "Usage:"
                for u in usages:
                    print "  %s %s"%(progname,u)
                return 2
            return 0
    # Unknown command - show usage for all commands
    print "%s: Unrecognized command (%s)"%(progname, args[1])
    print ""
    print "Available commands are:"
    for (cmds, test, usages) in ro_command_usage:
        for u in usages:
            print "  %s %s"%(progname,u)
    return 2

def help(progname, args):
    """
    Display ro command help.  See also ro --help
    """
    print "Available commands are:"
    for (cmds, test, usages) in ro_command_usage:
        for u in usages:
            print "  %s %s"%(progname,u)
    helptext = (
        [ ""
        , "Supported annotation type names are: "
        , "\n".join([ "  %(name)s - %(description)s"%atype for atype in annotationTypes ])
        , ""
        , "See also:"
        , "  %(progname)s --help"
        , ""
        ])
    for h in helptext:
        print h%{'progname': progname}
    return 0

def config(progname, configbase, options, args):
    """
    Update RO repository access configuration
    """
    robase = os.path.realpath(options.robasedir)
    ro_config = {
        "robase":               getoptionvalue(robase,
                                "RO local base directory:       "),
        "rosrs_uri":            getoptionvalue(options.rosrs_uri,
                                "URI for ROSRS service:         "),
        "rosrs_access_token":   getoptionvalue(options.rosrs_access_token,
                                "Access token for ROSRS service:"),
        "username":             getoptionvalue(options.username,
                                "Name of research object owner: "),
        "useremail":            getoptionvalue(options.useremail,
                                "Email address of owner:        "),
        # Built-in annotation types and prefixes
        "annotationTypes":      annotationTypes,
        "annotationPrefixes":   annotationPrefixes
        }
    ro_config["robase"] = os.path.abspath(ro_config["robase"])
    if options.verbose:
        print "ro config -b %(robase)s"%ro_config
        print "          -r %(rosrs_uri)s"%ro_config
        print "          -t %(rosrs_access_token)s"%ro_config
        print "          -n %(username)s -e %(useremail)s"%ro_config
    ro_utils.writeconfig(configbase, ro_config)
    if options.verbose:
        print "ro configuration written to %s"%(os.path.abspath(configbase))
    return 0

def create(progname, configbase, options, args):
    """
    Create a new Research Object.

    ro create RO-name [ -d dir ] [ -i RO-ident ]
    """
    ro_options = {
        "roname":  getoptionvalue(args[2],  "Name of new research object: "),
        "rodir":   options.rodir or "",
        "roident": options.roident or ""
        }
    log.debug("cwd: "+os.getcwd())
    log.debug("ro_options: "+repr(ro_options))
    ro_options['roident'] = ro_options['roident'] or ro_utils.ronametoident(ro_options['roname'])
    # Read local ro configuration and extract creator
    ro_config = ro_utils.readconfig(configbase)
    timestamp = datetime.datetime.now().replace(microsecond=0)
    ro_options['rocreator'] = ro_config['username']
    ro_options['rocreated'] = timestamp.isoformat()
    ro_dir = ro_utils.ropath(ro_config, ro_options['rodir'])
    if not ro_dir:
        print ("%s: research object not in configured research object directory tree: %s"%
               (ro_utils.progname(args), ro_options['rodir']))
        return 1
    # Create directory for manifest
    if options.verbose:
        print "ro create \"%(roname)s\" -d \"%(rodir)s\" -i \"%(roident)s\""%ro_options
    manifestdir = os.path.join(ro_dir, ro_settings.MANIFEST_DIR)
    log.debug("manifestdir: "+manifestdir)
    try:
        os.makedirs(manifestdir)
    except OSError:
        if os.path.isdir(manifestdir):
            # Someone else created it...
            # See http://stackoverflow.com/questions/273192/
            #          python-best-way-to-create-directory-if-it-doesnt-exist-for-file-write
            pass
        else:
            # There was an error on creation, so make sure we know about it
            raise
    # Create manifest file
    # @@TODO: create in-memory graph and serialize that
    manifestfilename = os.path.join(manifestdir, ro_settings.MANIFEST_FILE)
    log.debug("manifestfilename: "+manifestfilename)
    manifest = (
        """<?xml version="1.0" encoding="utf-8"?>
        <rdf:RDF
          xml:base=".."
          xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
          xmlns:ro="http://purl.org/wf4ever/ro#"
          xmlns:ore="http://www.openarchives.org/ore/terms/"
          xmlns:ao="http://purl.org/ao/"
          xmlns:dcterms="http://purl.org/dc/terms/"
          xmlns:foaf="http://xmlns.com/foaf/0.1/"
        >
          <ro:ResearchObject rdf:about="">
            <dcterms:identifier>%(roident)s</dcterms:identifier>
            <dcterms:title>%(roname)s</dcterms:title>
            <dcterms:description>%(roname)s</dcterms:description>
            <dcterms:creator>%(rocreator)s</dcterms:creator>
            <dcterms:created>%(rocreated)s</dcterms:created>
            <!-- self-reference to include above details as annotation -->
            <ore:aggregates>
              <ro:AggregatedAnnotation>
                <ro:annotatesAggregatedResource rdf:resource="" />
                <ao:body rdf:resource=".ro/manifest.rdf" />
              </ro:AggregatedAnnotation>
            </ore:aggregates>
          </ro:ResearchObject>
        </rdf:RDF>
        """%ro_options)
    log.debug("manifest: "+manifest)
    manifestfile = open(manifestfilename, 'w')
    manifestfile.write(manifest)
    manifestfile.close()
    return 0

def add(progname, configbase, options, args):
    """
    Add files to a research object manifest

    ro add [ -d dir ] file
    ro add [ -d dir ] [-a] [directory]

    Use -a/--all to add subdirectories recursively

    If no file or directory specified, defaults to current directory.
    """
    ro_config = ro_utils.readconfig(configbase)
    ro_options = {
        "rodir":        options.rodir or "",
        "rofile":       args[2] if len(args) == 3 else ".",
        "recurse":      options.all,
        "recurseopt":   "-a" if options.all else ""
        }
    log.debug("ro_options: "+repr(ro_options))
    # Find RO root directory
    ro_dir = ro_root_directory(progname+" add", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Read and update manifest
    if options.verbose:
        print "ro add -d %(rodir)s %(recurseopt)s %(rofile)s"%ro_options
    rometa = ro_metadata(ro_config, ro_dir)
    rometa.addAggregatedResources(ro_options['rofile'], ro_options['recurse'])
    return 0

def status(progname, configbase, options, args):
    """
    Display status of a designated research object

    ro status [ -d dir ]
    """
    # Check command arguments
    ro_config = ro_utils.readconfig(configbase)
    ro_options = {
        "rodir":   options.rodir or "",
        }
    log.debug("ro_options: "+repr(ro_options))
    # Find RO root directory
    ro_dir = ro_root_directory(progname+" status", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Read manifest and display status
    if options.verbose:
        print "ro status -d \"%(rodir)s\""%ro_options
    rometa = ro_metadata(ro_config, ro_dir)
    rodict = rometa.getRoMetadataDict()
    print "Research Object status"
    print "  identifier:  %(roident)s, title: %(rotitle)s"%rodict
    print "  creator:     %(rocreator)s, created: %(rocreated)s"%rodict
    print "  path:        %(ropath)s"%rodict
    if rodict['rouri']:
        print "  uri:         %(rouri)s"%rodict
    print "  description: %(rodescription)s"%rodict
    return 0

def list(progname, configbase, options, args):
    """
    List contents of a designated research object

    ro list [ -a ] [ -d dir ]
    ro ls [ -a ] [ -d dir ]
    """
    # Check command arguments
    ro_config = ro_utils.readconfig(configbase)
    ro_options = {
        "rodir":   options.rodir or "",
        }
    log.debug("ro_options: "+repr(ro_options))
    # Find RO root directory
    ro_dir = ro_root_directory(progname+" list", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Scan directory tree and display files
    if options.verbose:
        print "ro list -d \"%(rodir)s\""%ro_options
    rofiles = MiscLib.ScanDirectories.CollectDirectoryContents(
                ro_dir, baseDir=os.path.abspath(ro_dir),
                listDirs=False, listFiles=True, recursive=True, appendSep=False)
    if not options.all:
        def notHidden(f):
            return re.match("\.|.*/\.", f) == None
        rofiles = filter(notHidden, rofiles)
    print "\n".join(rofiles)
    return 0

def annotate(progname, configbase, options, args):
    """
    Annotate a specified research object component

    ro annotate file attribute-name [ attribute-value ]
    ro link file attribute-name [ attribute-value ]
    """
    ro_config = ro_utils.readconfig(configbase)
    rodir = options.rodir or os.path.dirname(args[2])
    if len(args) == 3:
        # Using graph form
        ro_options = {
            # Usding graph annotation form
            "rofile":       args[2],
            "rodir":        rodir,
            "graph":        options.graph or None,
            "defaultType":  "resource" if args[1] == "link" else "string"
            }
    else:
        ro_options = {
            # Usding explicit annotation form
            "rofile":       args[2],
            "rodir":        rodir,
            "roattribute":  args[3],
            "rovalue":      args[4] if len(args) == 5 else None,
            "defaultType":  "resource" if args[1] == "link" else "string"
            }
    log.debug("ro_options: "+repr(ro_options))
    # Find RO root directory
    ro_dir = ro_root_directory(progname+" annotate", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Read and update manifest and annotations
    rometa = ro_metadata(ro_config, ro_dir)
    rofile = ro_uriutils.resolveFileAsUri(ro_options['rofile'])     # Relative to CWD
    if len(args) == 3:
        # Add existing graph as annotation
        if options.verbose:
            print "ro annotate -d %(rodir)s %(rofile)s -g %(graph)s "%(ro_options)
            #print "ro annotate -d %(rodir)s"%ro_options
            #print "ro annotate %(rofile)s"%ro_options
            #print "ro annotate -g %(graph)s"%ro_options
        rometa.addGraphAnnotation(rofile, ro_options['graph'])
    else:
        # Create new annotation graph
        if options.verbose:
            print "ro annotate -d %(rodir)s %(rofile)s %(roattribute)s \"%(rovalue)s\""%ro_options
        rometa.addSimpleAnnotation(rofile,
            ro_options['roattribute'], ro_options['rovalue'],
            ro_options['defaultType'])
    return 0

def annotations(progname, configbase, options, args):
    """
    Display annotations

    ro annotations [ file | -d dir ]
    """
    log.debug("annotations: progname %s, configbase %s, args %s"%
              (progname, configbase, repr(args)))
    ro_config  = ro_utils.readconfig(configbase)
    ro_file    = (args[2] if len(args) >= 3 else "")
    ro_options = {
        "rofile":       ro_file,
        "rodir":        options.rodir or os.path.dirname(ro_file)
        }
    log.debug("ro_options: "+repr(ro_options))
    if options.verbose:
        print "ro annotations -d \"%(rodir)s\" %(rofile)s "%ro_options
    ro_dir = ro_root_directory(progname+" annotations", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Enumerate and display annotations
    rometa = ro_metadata(ro_config, ro_dir)
    if ro_options['rofile']:
        rofile = ro_uriutils.resolveFileAsUri(ro_options['rofile'])  # Relative to CWD
        log.debug("Annotations for %s"%str(rofile))
        annotations = rometa.getFileAnnotations(rofile)
    else:
        annotations = rometa.getAllAnnotations()
    if options.debug:
        log.debug("---- Annotations:")
        for a in annotations:
            log.debug("  %s"%repr(a))
        log.debug("----")
    rometa.showAnnotations(annotations, sys.stdout)
    return 0

def push(progname, configbase, options, args):
    """
    Push all or selected ROs and their resources to ROSRS

    ro push [ -d <dir> ] [ -f ] [ -r <rosrs_uri> ] [ -t <access_token> ]
    """
    ro_config = ro_utils.readconfig(configbase)
    ro_options = {
        "rodir":          options.rodir or None,
        "rosrs_uri":      options.rosrs_uri or getoptionvalue(ro_config['rosrs_uri'], "URI for ROSRS service:          "),
        "rosrs_access_token": options.rosrs_access_token or getoptionvalue(ro_config['rosrs_access_token'],
                                                                                      "Access token for ROSRS service: "),
        "force":          options.force
        }
    log.debug("ro_options: "+repr(ro_options))
    if options.verbose:
        print "ro push -d %(rodir)s -r %(rosrs_uri)s -t %(rosrs_access_token)s"%ro_options
    api = RosrsApi(ro_options['rosrs_uri'], ro_options['rosrs_access_token'])
    back = ResourceSync(api)
    if not ro_options['rodir']:
        try:
            (sent, deleted) = back.pushAllResourcesInWorkspace(ro_config['robase'], True, ro_options['force'])
        except Exception as e:
            print "Could not push all Research Objects: %s" % e
            return 1
    else:
        try:
            roDir = ro_utils.ropath(ro_config, ro_options['rodir'])
            if not roDir:
                raise Exception("%(rodir)s is not a valid RO folder" % ro_options)
            (sent, deleted) = back.pushAllResources(roDir, True, ro_options['force'])
        except Exception as e:
            print "Could not push Research Object in folder %s: %s" % (roDir, e)
            return 1
    if not options.verbose:
        print "%d files updated, %d files deleted" % (len(sent), len(deleted))
    else:
        for s in sent:
            print "Updated: %s" % s
        for d in deleted:
            print "Deleted: %s" % d
    return 0

def checkout(progname, configbase, options, args):
    """
    Checkout a RO from ROSRS

    ro checkout [ <RO-identifier> ] [-d <dir> ] [ -r <rosrs_uri> ] [ -t <access_token> ]
    """
    ro_config = ro_utils.readconfig(configbase)
    ro_options = {
        "roident":        args[2] if len(args) > 2 else None,
        "rodir":          options.rodir or "",
        "rosrs_uri":      options.rosrs_uri or getoptionvalue(ro_config['rosrs_uri'], "URI for ROSRS service:          "),
        "rosrs_access_token": options.rosrs_access_token or getoptionvalue(ro_config['rosrs_access_token'],
                                                                                      "Access token for ROSRS service: "),
        "force":          options.force
        }
    log.debug("ro_options: "+repr(ro_options))
    if options.verbose:
        print "ro checkout %(roident)s %(rodir)s %(rosrs_uri)s %(rosrs_access_token)s"%ro_options
    api = RosrsApi(ro_options['rosrs_uri'], ro_options['rosrs_access_token'])
    if (ro_options["roident"]):
        roident = ro_options["roident"]
        print "Checking out %s..." % roident
        rodir = os.path.join(ro_options['rodir'], roident)
        try:
            verzip = api.getRoAsZip(roident)
            __unpackZip(verzip, rodir, options.verbose)
        except urllib2.URLError as e:
            print "Could not checkout %s: %s" % (roident, e)
    else:
        ros = api.getRos()
        print "Checking out %d Research Objects..." % len(ros)
        for ro in ros:
            roident = os.path.basename(os.path.dirname(ro))
            print "---"
            print "Checking out %s..." % roident
            rodir = os.path.join(ro_options['rodir'], roident)
            verzip = api.getRoAsZipByUrl(ro)
            __unpackZip(verzip, rodir, options.verbose)
    return 0

def __unpackZip(verzip, rodir, verbose):
    zipfile = ZipFile(verzip)

    if verbose:
        for l in zipfile.namelist():
            print os.path.join(rodir, l)

    if not os.path.exists(rodir) or not os.path.isdir(rodir):
        os.mkdir(rodir)
    zipfile.extractall(rodir)

    print "%d files checked out" % len(zipfile.namelist())
    return 0

def evaluate(progname, configbase, options, args):
    """
    Evaluate RO

    ro evaluate checklist [ -d <dir> ] <minim> <purpose> [ <target> ]"
    """
    log.debug("evaluate: progname %s, configbase %s, args %s"%
              (progname, configbase, repr(args)))
    ro_config  = ro_utils.readconfig(configbase)
    ro_options = (
        { "rodir":        options.rodir or ""
        , "function":     args[2]
        })
    log.debug("ro_options: "+repr(ro_options))
    ro_dir = ro_root_directory(progname+" annotations", ro_config, ro_options['rodir'])
    if not ro_dir: return 1
    # Evaluate...
    if ro_options["function"] == "checklist":
        if len(args) not in [5,6]:
            print ("%s evaluate checklist: wrong number of arguments provided"%(progname))
            print ("Usage: %s evaluate checklist [ -d <dir> ] [ -a | -l <level> ] <minim> <purpose> [ <target> ]"%(progname))
            return 1
        levels = ["summary", "must", "should", "may", "full"]
        if options.level not in ["summary", "must", "should", "may", "full"]:
            print ("%s evaluate checklist: invalid reporting level %s, must be one of %s"%
                    (progname, options.level, repr(levels)))
            return 1
        ro_options["minim"]   = ((len(args) > 3) and args[3]) or "minim.rdf"
        ro_options["purpose"] = ((len(args) > 4) and args[4]) or "create"
        ro_options["target"]  = ((len(args) > 5) and args[5]) or "."
        if options.verbose:
            print "ro evaluate %(function)s -d \"%(rodir)s\" %(minim)s %(purpose)s %(target)s"%ro_options
        rometa = ro_metadata(ro_config, ro_dir)
        (minimgraph, evalresult) = ro_eval_minim.evaluate(rometa,
            ro_options["minim"], ro_options["target"], ro_options["purpose"])
        if options.verbose:
            print "== Evaluation result =="
            print json.dumps(evalresult, indent=2)
        ro_eval_minim.format(evalresult,
            { "detail" : "full" if options.all else options.level },
            sys.stdout)
    # elif ... other functions here
    else:
        print ("%s evaluate: unrecognized function provided (%s)"%(progname, ro_options["function"]))
        print ("Usage:")
        print ("  %s evaluate checklist [ -d <dir> ] [ -a | -l <level> ] <minim> <purpose> [ <target> ]"%(progname))
        return 1
    return 0

# End.



#!/usr/bin/python

"""
Research Object manager tests support module:  provides common functions used by multiple test suites.
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

if __name__ == "__main__":
    # Add main project directory and ro manager directories to python path
    sys.path.append("../..")
    sys.path.append("..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro, ro_utils, ro_manifest

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

class TestROSupport(unittest.TestCase):
    """
    Test basic ro commands
    """
    def setUp(self):
        super(TestROSupport, self).setUp()
        self.save_cwd = os.getcwd()
        self.outstr = StringIO.StringIO()
        return

    def tearDown(self):
        self.outstr.close()
        super(TestROSupport, self).tearDown()
        os.chdir(self.save_cwd)
        return

    def getConfigDir(self, testbase):
        return os.path.join(testbase, ro_test_config.CONFIGDIR)

    def getRoBaseDir(self, testbase):
        return os.path.join(testbase, ro_test_config.ROBASEDIR)

    def setupTestBaseConfig(self, testbase):
        """
        Test helper creates RO config for designated test base directory
        and returns configuration and base RO storage directories. 
        """
        # @@refactor to use method from rocommand
        configdir = self.getConfigDir(testbase)
        robasedir = self.getRoBaseDir(testbase)
        ro_utils.resetconfig(configdir)
        args = [
            "ro", "config",
            "-b", robasedir,
            "-r", ro_test_config.ROSRS_URI,
            "-t", ro_test_config.ROSRS_ACCESS_TOKEN,
            "-n", ro_test_config.ROBOXUSERNAME,
            "-e", ro_test_config.ROBOXEMAIL
            ]
        with SwitchStdout(self.outstr):
            status = ro.runCommand(configdir, robasedir, args)
            assert status == 0
        self.assertEqual(self.outstr.getvalue().count("ro config"), 0)
        return (configdir, robasedir)

    def createRoFixture(self, testbase, src, robase, roname):
        """
        Create test fixture research object - this is a set of directories
        and files that will be used as a research object, but not actually
        creating the research object specific structures.
        
        Returns name of research object directory
        """
        if testbase != "" and not testbase.endswith("/"): testbase += "/"
        rodir = testbase + robase + "/" + roname
        manifestdir  = rodir+"/"+ro_test_config.ROMANIFESTDIR
        manifestfile = manifestdir+"/"+ro_test_config.ROMANIFESTFILE
        shutil.rmtree(rodir, ignore_errors=True)
        shutil.copytree(testbase+src, rodir)
        # Confirm non-existence of manifest directory
        self.assertTrue(os.path.exists(rodir), msg="checking copied RO directory")
        self.assertFalse(os.path.exists(manifestdir), msg="checking copied RO manifest dir")
        return rodir

    def checkRoFixtureManifest(self, rodir):
        """
        Test for existence of manifest in RO fixture.
        """
        manifestdir  = rodir+"/"+ro_test_config.ROMANIFESTDIR
        manifestfile = manifestdir+"/"+ro_test_config.ROMANIFESTFILE
        self.assertTrue(os.path.exists(manifestdir), msg="checking created RO manifest dir")
        self.assertTrue(os.path.exists(manifestfile), msg="checking created RO manifest file")
        return

    def checkManifestContent(self, rodir, roname, roident):
        manifest = ro_manifest.readManifest(rodir)
        self.assertEqual(manifest['roident'],       roident, "RO identifier")
        self.assertEqual(manifest['rotitle'],       roname,  "RO title")
        self.assertEqual(manifest['rocreator'],     ro_test_config.ROBOXUSERNAME, "RO creator")
        # See: http://stackoverflow.com/questions/969285/
        #      how-do-i-translate-a-iso-8601-datetime-string-into-a-python-datetime-object
        rocreated = datetime.datetime.strptime(manifest['rocreated'], "%Y-%m-%dT%H:%M:%S")
        timenow   = datetime.datetime.now().replace(microsecond=0)
        rodelta   = timenow-rocreated
        self.assertTrue(rodelta.seconds<=1, 
            "Unexpected created datetime: %s, expected about %s"%
                (manifest['rocreated'],timenow.isoformat()))
        self.assertEqual(manifest['rodescription'], roname,  "RO name")
        return

    def checkManifestGraph(self, rodir, rograph):
        """
        Check manifest file contains all statements from supplied graph
        """
        m_graph = ro_manifest.readManifestGraph(rodir)
        for (s,p,o) in rograph:
            if isinstance(s, rdflib.BNode): s = None 
            if isinstance(o, rdflib.BNode): o = None
            self.assertIn((s,p,o), m_graph, "Not found in manifest: "+repr((s, p, o)))
        return

    def checkTargetGraph(self, targetgraph, expectgraph, msg="Not found in target graph"):
        """
        Check target graph contains all statements from supplied graph
        """
        for (s,p,o) in expectgraph:
            if isinstance(s, rdflib.BNode): s = None 
            if isinstance(o, rdflib.BNode): o = None
            self.assertIn((s,p,o), targetgraph, msg+": "+repr((s, p, o)))
        return

    def deleteRoFixture(self, rodir):
        """
        Delete test fixture research object
        """
        shutil.rmtree(rodir, ignore_errors=True)
        return

    def createTestRo(self, testbase, src, roname, roident):
        """
        Create test research object
        
        Returns name of research object directory
        """
        # @@refactor to use method from rocommand
        rodir = self.createRoFixture(testbase, src, ro_test_config.ROBASEDIR, ro_utils.ronametoident(roname))
        args = [
            "ro", "create", roname,
            "-v", 
            "-d", rodir,
            "-i", roident,
            ]
        with SwitchStdout(self.outstr):
            configdir = self.getConfigDir(testbase)
            robasedir = self.getRoBaseDir(testbase)
            status = ro.runCommand(configdir, robasedir, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        # @@TODO: is this helpful? Reset output stream buffer closed
        self.outstr = StringIO.StringIO()
        return rodir

    def populateTestRo(self, testbase, rodir):
        """
        Create test research object
        
        Returns name of research object directory
        """
        # @@refactor to use method from rocommand
        args = [
            "ro", "add", "-v", "-a",
            "-d", rodir,
            rodir
            ]
        with SwitchStdout(self.outstr):
            configdir = self.getConfigDir(testbase)
            robasedir = self.getRoBaseDir(testbase)
            status = ro.runCommand(configdir, robasedir, args)
        outtxt = self.outstr.getvalue()
        assert status == 0, outtxt
        return rodir

    def deleteTestRo(self, rodir):
        """
        Delete test research object
        """
        self.deleteRoFixture(rodir)
        return

    # UnitTest support from Python 2.7

    def assertRegexpMatches(self, text, expected_regexp, msg=None):
        """Fail the test unless the text matches the regular expression."""
        if isinstance(expected_regexp, basestring):
            expected_regexp = re.compile(expected_regexp)
        if not expected_regexp.search(text):
            msg = msg or "Regexp didn't match"
            msg = '%s: %r not found in %r' % (msg, expected_regexp.pattern, text)
            raise self.failureException(msg)

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestROSupport, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestROSupport.log", getTestSuite, sys.argv)

# End.



import uritemplate
import simplejson
import sys

f = file("testdata.json")
testdata = simplejson.load(f)

for name, testsuite in testdata.iteritems():
  print name
  vars = testsuite['variables']
  testcases= testsuite['testcases']

  for testcase in testcases:
    template = testcase[0]
    expected = testcase[1]
    actual = uritemplate.expand(template, vars)
    sys.stdout.write(".") 
    if expected != actual:
      print "Template %s expected to expand to %s, got %s instead" % (template, expected, actual)
      assert 0
  print



# __init__.py
#__version__ = "0.2.1"       # Initial version with installation package
#__version__ = "0.2.2"       # Updated README documentation for PyPI page
#__version__ = "0.2.3"       # Experimenting with distribution options
#__version__ = "0.2.4"       # Added MANIFEST.in so that data files are part of sdist
__version__ = "0.2.5"       # Drop references to lpod-show from test suite



#!/usr/bin/env python

"""
RO manager command parser and dispatcher
"""

import sys
import os
import os.path
import re
import codecs
import optparse
import logging

log = logging.getLogger(__name__)

# Make sure MiscLib can be found on path
if __name__ == "__main__":
    sys.path.append(os.path.join(sys.path[0],".."))

import ro_settings
import ro_command
import ro_utils

def run(configbase, options, args):
    status = 0
    progname = ro_utils.progname(args)
    if len(args) < 2:
        print "%s No command given"%(progname)
        print "Enter '%s help' to show a list of commands"
        status = 2
    else:
        status = ro_command.check_command_args(progname, options, args)
    if status != 0: return status
    #@@TODO: refactor to use command/usage table in rocommand for dispatch
    if args[1] == "help":
        status = ro_command.help(progname, args)
    elif args[1] == "config":
        status = ro_command.config(progname, configbase, options, args)
    elif args[1] == "create":
        status = ro_command.create(progname, configbase, options, args)
    elif args[1] == "add":
        status = ro_command.add(progname, configbase, options, args)
    elif args[1] == "status":
        status = ro_command.status(progname, configbase, options, args)
    elif args[1] == "list" or args[1] == "ls":
        status = ro_command.list(progname, configbase, options, args)
    elif args[1] in ["annotate","link"]:
        status = ro_command.annotate(progname, configbase, options, args)
    elif args[1] == "annotations":
        status = ro_command.annotations(progname, configbase, options, args)
    elif args[1] == "evaluate" or args[1] == "eval":
        status = ro_command.evaluate(progname, configbase, options, args)
    elif args[1] == "push":
        status = ro_command.push(progname, configbase, options, args)
    elif args[1] == "checkout":
        status = ro_command.checkout(progname, configbase, options, args)
    else:
        print "%s: unrecognized command: %s"%(progname,args[1])
        status = 2
    return status

def parseCommandArgs(argv):
    """
    Parse command line arguments

    prog -- program name from command line
    argv -- argument list from command line

    Returns a pair consisting of options specified as returned by
    OptionParser, and any remaining unparsed arguments.
    """
    # create a parser for the command line options
    parser = optparse.OptionParser(
                usage="%prog [options] command [args...]\n\n",
                version="%prog "+ro_settings.VERSION)
    # version option
    parser.add_option("-a", "--all",
                      action="store_true",
                      dest="all",
                      default=False,
                      help="All, list all files, depends on the context")
    parser.add_option("-b", "--ro-base",
                      dest="robasedir",
                      help="Base of local directory tree used for ROs")
    parser.add_option("-d", "--ro-directory",
                      dest="rodir",
                      help="Directory of Research Object to process (defaults to current directory)")
    parser.add_option("-e", "--user-email",
                      dest="useremail",
                      help="Email address of research objects owner")
    parser.add_option("-f", "--force",
                      action="store_true",
                      dest="force",
                      default=False,
                      help="Force, depends on the context")
    parser.add_option("-g", "--graph",
                      dest="graph",
                      help="Name of existing RDF graph used for annotation")
    parser.add_option("-i", "--ro-identifier",
                      dest="roident",
                      help="Identifier of Research Object (defaults to value based on name)")
    parser.add_option("-l", "--report-level",
                      dest="level",
                      default="may",
                      help="Level of report detail to generate (summary, must, should, may or full)")
    parser.add_option("-n", "--user-name",
                      dest="username",
                      help="Full name of research objects owner")
    parser.add_option("-r", "--rosrs-uri",
                      dest="rosrs_uri",
                      help="URI of ROSRS service")
    parser.add_option("-t", "--rosrs-access-token",
                      dest="rosrs_access_token",
                      help="ROSRS access token")
    parser.add_option("-v", "--verbose",
                      action="store_true",
                      dest="verbose",
                      default=False,
                      help="display verbose output")
    parser.add_option("--debug",
                      action="store_true",
                      dest="debug",
                      default=False,
                      help="display debug output")
    # parse command line now
    (options, args) = parser.parse_args(argv)
    if len(args) < 2: parser.error("No command present")
    return (options, args)

def runCommand(configbase, robase, argv):
    """
    Run program with supplied configuration base directory, Base directory
    from which to start looking for research objects, and arguments.

    This is called by main function (below), and also by test suite routines.

    Returns exit status.
    """
    (options, args) = parseCommandArgs(argv)
    if not options or options.debug:
        logging.basicConfig(level=logging.DEBUG)
    log.debug("runCommand: configbase %s, robase %s, argv %s"%(configbase, robase, repr(argv)))
    status = 1
    if options:
        status  = run(configbase, options, args)
    return status

def runMain():
    """
    Main program transfer function for setup.py console script
    """
    configbase = os.path.expanduser("~")
    robase = os.getcwd()
    return runCommand(configbase, robase, sys.argv)

if __name__ == "__main__":
    """
    Program invoked from the command line.
    """
    status = runMain()
    sys.exit(status)

#--------+---------+---------+---------+---------+---------+---------+---------+



'''
Created on 15-09-2011

@author: piotrhol
'''

from SyncRegistry import SyncRegistry
import mimetypes
import logging
from os.path import isdir, exists, join, basename
from os import listdir, walk
import pickle
from rocommand import ro_manifest
import urllib2

log = logging.getLogger(__name__)

class ResourceSync(object):
    '''
    classdocs
    '''
   
    REGISTRIES_FILE = ".registries.pickle"

    def __init__(self, rosrsSync):
        '''
        Constructor
        '''
        self.rosrsSync = rosrsSync
        mimetypes.init()
        
    def pushAllResourcesInWorkspace(self, srcDirectory, createRO = False, force = False):
        '''
        Scans all directories in srcDirectory as RO versions. 
        For each RO pushes all changes to ROSRS.
        
        If createRO is True, this method will try to create RO before pushing
        the resources. Otherwise, an exception will be raised if a directory is found without its
        corresponding RO in ROSRS.
        '''
        sentFiles = set()
        deletedFiles = set()
        for ro in listdir(srcDirectory):
            roDirectory = join(srcDirectory, ro)
            if isdir(roDirectory):
                try:
                    ro_manifest.readManifest(roDirectory)
                except:
                    log.debug("Not a valid RO: %s" % roDirectory)
                    continue
                (s, d) = self.pushAllResources(roDirectory, createRO, force)
                sentFiles = sentFiles.union(s)
                deletedFiles = deletedFiles.union(d)
            else:
                log.debug("%s is a file in workspace, it should probably be moved somewhere" % roDirectory)
        return (sentFiles, deletedFiles)
        
    def pushAllResources(self, roDirectory, createRO = False, force = False):
        '''
        Scans a given RO version directory for files that have been modified since last synchronization
        and pushes them to ROSRS. Modification is detected by checking modification times and checksums.
        '''
        roId = urllib2.quote(basename(roDirectory)) 
        ro_manifest.readManifest(roDirectory)
        if force:
            self.syncRegistries = dict()
        else:
            self.syncRegistries = self.__loadRegistries(roDirectory)
        if createRO:
            try:
                self.rosrsSync.postRo(roId)
            except:
                log.debug("Failed to create RO %s" % roId)
        sentFiles = self.__scanDirectories4Put(roId, roDirectory)
        deletedFiles = self.__scanRegistries4Delete(roId, roDirectory)
        self.__saveRegistries(roDirectory)
        return (sentFiles, deletedFiles)
    
    def __scanDirectories4Put(self, roId, roDirectory):
        sentFiles = set()
        manifest = join(roDirectory, ".ro/manifest.rdf")
        if self.__checkFile4Put(roId, roDirectory, manifest):
            sentFiles.add(manifest)
        for root, dirs, files in walk(roDirectory):
            for f in files:
                filepath = join(root, f)
                if filepath != join(roDirectory, self.REGISTRIES_FILE) and self.__checkFile4Put(roId, roDirectory, filepath):
                    sentFiles.add(filepath)
        return sentFiles
    
    def __checkFile4Put(self, roId, roDirectory, filepath):
        assert filepath.startswith(roDirectory)
        rosrsFilepath = filepath[len(roDirectory) + 1:]
        put = True
        if (filepath in self.syncRegistries):
            put = self.syncRegistries[filepath].hasBeenModified()
        if put:
            contentType = mimetypes.guess_type(filepath)[0]
            fileObject = open(filepath)
            log.debug("Put file %s" % filepath)
            self.rosrsSync.putFile(roId, rosrsFilepath, contentType, fileObject)
            self.syncRegistries[filepath] = SyncRegistry(filepath)
        return put
    
    def __scanRegistries4Delete(self, roId, roDirectory):
        deletedFiles = set()
        for r in self.syncRegistries.viewvalues():
            if r.filename.startswith(roDirectory):
                if not exists(r.filename):
                    log.debug("Delete file %s" % r.filename)
                    deletedFiles.add(r.filename)
                    rosrsFilepath = r.filename[len(roDirectory) + 1:]
                    try:
                        self.rosrsSync.deleteFile(roId, rosrsFilepath)
                    except:
                        log.debug("File %s did not exist in ROSRS" % r.filename)
        for f in deletedFiles:
            del self.syncRegistries[f]
        return deletedFiles
    
    def __loadRegistries(self, roDirectory):
        try:
            rf = open(join(roDirectory, self.REGISTRIES_FILE), 'r')
            return pickle.load(rf)
        except:
            return dict()
        
    def __saveRegistries(self, roDirectory):
        rf = open(join(roDirectory, self.REGISTRIES_FILE), 'w')
        pickle.dump(self.syncRegistries, rf)
        return
        



# ro_eval_minim.py

"""
Research Object evaluation withj respoect to a MINIM description
"""

#import sys
#import os
#import os.path
#import urlparse
import re
import subprocess
import logging

log = logging.getLogger(__name__)

import rdflib
#import rdflib.namespace
#from rdflib import URIRef, Namespace, BNode
#from rdflib import Literal
from uritemplate import uritemplate

from rocommand.ro_uriutils   import isLiveUri
from rocommand.ro_namespaces import RDF, RDFS, ORE
from rocommand.ro_metadata   import ro_metadata
import ro_minim
from ro_minim import MINIM

def evaluate(rometa, minim, target, purpose):
    """
    Evaluate a RO against a minimuminformation model for a particular
    purpose with respect to a particular targetresource.

    rometa      is an ro_metadata object used to access the RO being evaluated
    minim       is a URI-reference (relative to the RO, or absolute) of the
                minim description to be used.
    target      is a URI-reference (relative to the RO, or absolute) of a 
                target resource with respect to which the evaluation is
                performed.
    purpose     is a string that identifies a purpose w.r.t. the target for
                which completeness will be evaluated.
                
    'target' and 'purpose' are ued together to select a particular minim Model
    that will be used for the evaluation.  For example, to evaluate whether an 
    RO is sufficiently complete to support creation (a purpose) of a specified
    output file (a target).
    
    There are two main steps to the evaluation process:
    1. locate the minim model constraint for the target resource and purpose
    2. evaluate the RO against the selected model.
    
    The result indicates a summary and details of the analysis; e.g.
    { 'summary':       [MINIM.fullySatisfies, MINIM.nominallySatisfies, MINIM.minimallySatisfies]
    , 'missingMust':   []
    , 'missingShould': []
    , 'missingMay':    []
    , 'rouri':          rouri
    , 'minimuri':       minim
    , 'target':         target
    , 'purpose':        purpose
    , 'constrainturi':  constraint['uri']
    , 'modeluri':       model['uri']
    }
    """
    # Locate the constraint model requirements
    rouri        = rometa.getRoUri()
    minimuri     = rometa.getComponentUri(minim)
    minimgraph   = ro_minim.readMinimGraph(minimuri)
    constraint   = ro_minim.getConstraint(minimgraph, rouri, target, purpose)
    assert constraint != None, "Missing minim:Constraint for target %s, purpose %s"%(target, purpose)
    model        = ro_minim.getModel(minimgraph, constraint['model'])
    assert model != None, "Missing minim:Model for target %s, purpose %s"%(target, purpose)
    requirements = ro_minim.getRequirements(minimgraph, model['uri'])
    # Evaluate the individual model requirements
    reqeval = []
    for r in requirements:
        if 'datarule' in r:
            # @@TODO: factor to separate function?
            satisfied = rometa.roManifestContains( (rouri, ORE.aggregates, r['datarule']['aggregates']) )
            reqeval.append((r,satisfied,{}))
            log.debug("- %s: %s"%(repr((rouri, ORE.aggregates, r['datarule']['aggregates'])), satisfied))
        elif 'softwarerule' in r:
            # @@TODO: factor to separate function
            cmnd = r['softwarerule']['command']
            resp = r['softwarerule']['response']
            log.debug("softwarerule: %s -> %s"%(cmnd,resp))
            out = unicode(subprocess.check_output(cmnd.split(), stderr=subprocess.STDOUT))
            exp = re.compile(resp)
            satisfied = exp.match(out)
            reqeval.append((r,satisfied,{}))
            log.debug("- Software %s: response %s,  satisfied %s"%(cmnd, resp, "OK" if satisfied else "Fail"))
        elif 'contentmatchrule' in r:
            (satisfied, bindings) = evalContentMatch(rometa, r['contentmatchrule'])
            reqeval.append((r,satisfied,bindings))
            log.debug("- ContentMatch: rule %s, bindings %s, satisfied %s"%
                      (repr(r['contentmatchrule']), repr(bindings), "OK" if satisfied else "Fail"))
        else:
            raise ValueError("Unrecognized requirement rule: %s"%repr(r.keys()))
    # Evaluate overall satisfaction of model
    sat_levels = (
        { 'MUST':   MINIM.minimallySatisfies
        , 'SHOULD': MINIM.nominallySatisfies
        , 'MAY':    MINIM.fullySatisfies
        })
    eval_result = (
        { 'summary':        []
        , 'missingMust':    []
        , 'missingShould':  []
        , 'missingMay':     []
        , 'satisfied':      []
        , 'rouri':          rouri
        , 'minimuri':       minimuri
        , 'target':         target
        , 'purpose':        purpose
        , 'constrainturi':  constraint['uri']
        , 'modeluri':       model['uri']
        })
    for (r, satisfied, binding) in reqeval:
        if satisfied:
            eval_result['satisfied'].append((r, binding))
        else:
            if r['level'] == "MUST":
                eval_result['missingMust'].append((r, binding))
                sat_levels['MUST']   = None
                sat_levels['SHOULD'] = None
                sat_levels['MAY']    = None
            elif r['level'] == "SHOULD":
                eval_result['missingShould'].append((r, binding))
                sat_levels['SHOULD'] = None
                sat_levels['MAY']    = None
            elif r['level'] == "MAY":
                eval_result['missingMay'].append((r, binding))
                sat_levels['MAY'] = None
    eval_result['summary'] = [ sat_levels[k] for k in sat_levels if sat_levels[k] ]
    return (minimgraph, eval_result)

def evalContentMatch(rometa, rule):
    querytemplate = """
        PREFIX rdf:        <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX rdfs:       <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX owl:        <http://www.w3.org/2002/07/owl#>
        PREFIX xsd:        <http://www.w3.org/2001/XMLSchema#>
        PREFIX xml:        <http://www.w3.org/XML/1998/namespace>
        PREFIX ro:         <http://purl.org/wf4ever/ro#>
        PREFIX wfprov:     <http://purl.org/wf4ever/wfprov#>
        PREFIX wfdesc:     <http://purl.org/wf4ever/wfdesc#>
        PREFIX rdfg:       <http://www.w3.org/2004/03/trix/rdfg-1/>
        PREFIX ore:        <http://www.openarchives.org/ore/terms/>
        PREFIX ao:         <http://purl.org/ao/>
        PREFIX dcterms:    <http://purl.org/dc/terms/>
        PREFIX foaf:       <http://xmlns.com/foaf/0.1/>

        %(queryverb)s
        {
          %(querypattern)s
        }
        """
    satisfied     = True
    simplebinding = {}
    if rule['forall']:
        exists   = rule['exists']
        template = rule['template']
        islive   = rule['islive']
        assert (exists or template or islive), (
            "minim:forall construct requires "+
            "minim:aggregatesTemplate, minim:isLiveTemplate and/or minim:exists value")
        queryparams = (
            { 'queryverb': "SELECT * WHERE"
            , 'querypattern': rule['forall']
            })
        query = querytemplate%queryparams
        resp  = rometa.queryAnnotations(query)
        for binding in resp:
            satisfied = False
            # Extract keys and values from query result
            simplebinding = {}
            for k in binding:
                ###print "key: "+repr(k) 
                if not isinstance(k,rdflib.BNode):
                    simplebinding[str(k)] = str(binding[k])
                    # @@TODO remove this when rdflib bug resolved 
                    if str(k) in ['if', 'of'] and str(binding[k])[:5] not in ["file:","http:"]:
                        # Houston, we have a problem...
                        agraph = rometa.roannotations
                        print "--------------------"
                        print "Graph: "+agraph.serialize(format="xml")
                        print "Query: "+query
                        print "Response bindings: "+repr(resp)
                        print "--------------------"
                        assert False, "Aborted"
            if exists:
                # existence query against forall results
                existsparams = (
                    { 'queryverb': "ASK"
                    , 'querypattern': exists
                    })
                query = querytemplate%existsparams
                log.debug("***** evalContentMatch RO test exists: \nquery: %s \nbinding: %s)"%(query, repr(binding)))
                satisfied = rometa.queryAnnotations(query,initBindings=binding)
            if template:
                # Construct URI for file from template
                # Uses code copied from http://code.google.com/p/uri-templates
                fileref = uritemplate.expand(template, simplebinding)
                fileuri = rometa.getComponentUri(fileref)
                # Test if URI is aggregated
                log.debug("evalContentMatch RO aggregates %s (%s)"%(fileref, str(fileuri)))
                satisfied = rometa.roManifestContains( (rometa.getRoUri(), ORE.aggregates, fileuri) )
            if islive:
                # Construct URI for file from template
                # Uses code copied from http://code.google.com/p/uri-templates
                fileref = uritemplate.expand(islive, simplebinding)
                fileuri = rometa.getComponentUri(fileref)
                # Test if URI is live (accessible)
                log.debug("evalContentMatch RO islive %s (%s)"%(fileref, str(fileuri)))
                satisfied = isLiveUri(fileuri)
            log.debug("evalContentMatch (forall) RO satisfied %s"%(satisfied))
            if not satisfied: break
    elif rule['exists']:
        queryparams = (
            { 'queryverb': "ASK"
            , 'querypattern': rule['exists']
            })
        query = querytemplate%queryparams
        satisfied = rometa.queryAnnotations(query)
    else:
        raise ValueError("Unrecognized content match rule: %s"%repr(rule))
    return (satisfied,simplebinding)

def format(eval_result, options, ostr):
    """
    Formats a completeness evaluation report, and writes it to the supplied stream.
    
    eval_result is the result of evaluation from ro_eval_completeness.evaluate
    options     a dictionary that provides options to control the formatting (see below)
    ostr        is a stream to which the formatted result is written

    options currently has just one field:
    options['detail'] = "summary", "must", "should", "may" or "full"
    """
    s_any     = ["full", "may", "should", "must", "summary"]
    s_must    = ["full", "may", "should", "must"]
    s_should  = ["full", "may", "should"]
    s_may     = ["full", "may"]
    s_full    = ["full"]
    def put(detail, line):
        if options['detail'] in detail:
            ostr.write(line)
            ostr.write("\n")
        return
    put(s_any, "Research Object %(rouri)s:"%eval_result)
    summary_text= ( "Fully complete"     if MINIM.fullySatisfies     in eval_result['summary'] else
                    "Nominally complete" if MINIM.nominallySatisfies in eval_result['summary'] else
                    "Minimally complete" if MINIM.minimallySatisfies in eval_result['summary'] else
                    "Incomplete")
    put(s_any, summary_text+" for %(purpose)s of resource %(target)s"%(eval_result))
    if eval_result['missingMust']:
        put(s_must, "Unsatisfied MUST requirements:")
        for m in eval_result['missingMust']:
            put(s_must, "  "+formatRule(False, *m))
    if eval_result['missingShould']:
        put(s_should, "Unsatisfied SHOULD requirements:")
        for m in eval_result['missingShould']:
            put(s_should, "  "+formatRule(False, *m))
    if eval_result['missingMay']:
        put(s_may, "Unsatisfied MAY requirements:")
        for m in eval_result['missingMay']:
            put(s_may, "  "+formatRule(False, *m))
    if eval_result['satisfied']:
        put(s_full, "Satisfied requirements:")
        for m in eval_result['satisfied']:
            put(s_full, "  "+formatRule(True, *m))
    put(s_full, "Research object URI:     %(rouri)s"%(eval_result))
    put(s_full, "Minimum information URI: %(minimuri)s"%(eval_result))
    return

def formatRule(satisfied, rule, bindings):
    """
    Format a rule for a missing/satisfied report
    """
    templateindex = "showpass" if satisfied else "showfail"
    if 'datarule' in rule:
        ruledict = rule['datarule']
        templatedefault = "Aggregates resource %(aggregates)s"
    elif 'softwarerule' in rule:
        ruledict = rule['softwarerule']
        templatedefault = "Environment '%(command)s' matches '%(response)s'"
    elif 'contentmatchrule' in rule:
        ruledict = rule['contentmatchrule']
        if ruledict['exists']:
            templatedefault = "Match for %(exists)s"
        else:
            templatedefault = "Aggregate %(template)s for matching %(forall)s"
    else:
        ruledict = { 'rule': repr(rule), 'show': None, templateindex: None }
        templatedefault = "Unrecognized rule: %(rule)s"
    template = ruledict[templateindex] or ruledict["show"] or templatedefault
    bindings.update(ruledict)
    return template%bindings

# End.



#!/usr/bin/python

"""
Module to test RO manager annotation support utilities

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro_settings
from rocommand import ro_manifest
from rocommand import ro_annotation
from rocommand.ro_namespaces import RDF, RDFS, RO, ORE, DCTERMS, ROTERMS

from TestConfig import ro_test_config
from StdoutContext import SwitchStdout

import TestROSupport

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

# Local ro_config for testing
ro_config = {
    "annotationTypes":      ro_annotation.annotationTypes,
    "annotationPrefixes":   ro_annotation.annotationPrefixes
    }

cwd        = os.getcwd()
robase     = ro_test_config.ROBASEDIR
robase_abs = os.path.abspath(ro_test_config.ROBASEDIR)

class TestAnnotationUtils(TestROSupport.TestROSupport):
    """
    Test ro annotation support functions
    """
    def setUp(self):
        super(TestAnnotationUtils, self).setUp()
        return

    def tearDown(self):
        super(TestAnnotationUtils, self).tearDown()
        return

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testGetFileUri(self):
        self.assertEquals(ro_manifest.getFileUri("/example/a/b.txt"),
                          rdflib.URIRef("file:///example/a/b.txt"))
        self.assertEquals(ro_manifest.getFileUri("a/b.txt"),
                          rdflib.URIRef("file://%s/a/b.txt"%(cwd)))
        self.assertEquals(ro_manifest.getFileUri("/example/ro/dir/a/b/d/"),
                          rdflib.URIRef("file:///example/ro/dir/a/b/d/"))
        self.assertEquals(ro_manifest.getFileUri("a/b/d/"),
                          rdflib.URIRef("file://%s/a/b/d/"%(cwd)))
        return

    def testGetUriFile(self):
        self.assertEquals(ro_manifest.getUriFile(rdflib.URIRef("file:///example/a/b.txt")), "/example/a/b.txt")
        self.assertEquals(ro_manifest.getUriFile(rdflib.URIRef("/example/a/b.txt")), "/example/a/b.txt")
        self.assertEquals(ro_manifest.getUriFile(rdflib.URIRef("a/b.txt")), "a/b.txt")
        return

    def testGetRoUri(self):
        self.assertEquals(ro_manifest.getRoUri("/example/ro/dir"), rdflib.URIRef("file:///example/ro/dir/"))
        self.assertEquals(ro_manifest.getRoUri("/example/ro/dir/"), rdflib.URIRef("file:///example/ro/dir/"))
        self.assertEquals(ro_manifest.getRoUri("ro/dir"), rdflib.URIRef("file://%s/ro/dir/"%(cwd)))
        self.assertEquals(ro_manifest.getRoUri("ro/dir/"), rdflib.URIRef("file://%s/ro/dir/"%(cwd)))
        self.assertEquals(ro_manifest.getRoUri(robase+"/ro/dir"), rdflib.URIRef("file://%s/ro/dir/"%(robase_abs)))
        self.assertEquals(ro_manifest.getRoUri(robase+"/ro/dir/"), rdflib.URIRef("file://%s/ro/dir/"%(robase_abs)))
        return

    def testGetComponentUri(self):
        self.assertEquals(ro_manifest.getComponentUri("/example/ro/dir", "a/b.txt"),
                          rdflib.URIRef("file:///example/ro/dir/a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUri("/example/ro/dir/", "a/b.txt"),
                          rdflib.URIRef("file:///example/ro/dir/a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUri("ro/dir", "a/b.txt"),
                          rdflib.URIRef("file://%s/ro/dir/a/b.txt"%(cwd)))
        self.assertEquals(ro_manifest.getComponentUri("ro/dir/", "a/b.txt"),
                          rdflib.URIRef("file://%s/ro/dir/a/b.txt"%(cwd)))
        self.assertEquals(ro_manifest.getComponentUri("/example/ro/dir", "a/b/d/"),
                          rdflib.URIRef("file:///example/ro/dir/a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUri("/example/ro/dir/", "a/b/d/"),
                          rdflib.URIRef("file:///example/ro/dir/a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUri("ro/dir", "a/b/d/"),
                          rdflib.URIRef("file://%s/ro/dir/a/b/d/"%(cwd)))
        self.assertEquals(ro_manifest.getComponentUri("ro/dir/", "a/b/d/"),
                          rdflib.URIRef("file://%s/ro/dir/a/b/d/"%(cwd)))
        return

    def testGetComponentUriRel(self):
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", "a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", "a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", "a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", "a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", ""),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", ""),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", ""),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", ""),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "/example/ro/dir/a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "/example/ro/dir/a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", "%s/ro/dir/a/b.txt"%(cwd)),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", "%s/ro/dir/a/b.txt"%(cwd)),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "/example/ro/dir/a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "/example/ro/dir/a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", "%s/ro/dir/a/b/d/"%(cwd)),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", "%s/ro/dir/a/b/d/"%(cwd)),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "/example/ro/dir/"),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "/example/ro/dir/"),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir", "%s/ro/dir/"%(cwd)),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("ro/dir/", "%s/ro/dir/"%(cwd)),
                          rdflib.URIRef(""))
        # Test supplied file: URI string
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "file:///example/ro/dir/a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "file:///example/ro/dir/a/b.txt"),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "file:///example/ro/dir/a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "file:///example/ro/dir/a/b/d/"),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir", "file:///example/ro/dir/"),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/", "file:///example/ro/dir/"),
                          rdflib.URIRef(""))
        # Test supplied file: URI
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir",
                                                         rdflib.URIRef("file:///example/ro/dir/a/b.txt")),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/",
                                                         rdflib.URIRef("file:///example/ro/dir/a/b.txt")),
                          rdflib.URIRef("a/b.txt"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir",
                                                         rdflib.URIRef("file:///example/ro/dir/a/b/d/")),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/",
                                                         rdflib.URIRef("file:///example/ro/dir/a/b/d/")),
                          rdflib.URIRef("a/b/d/"))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir",
                                                         rdflib.URIRef("file:///example/ro/dir/")),
                          rdflib.URIRef(""))
        self.assertEquals(ro_manifest.getComponentUriRel("/example/ro/dir/",
                                                         rdflib.URIRef("file:///example/ro/dir/")),
                          rdflib.URIRef(""))
        return

    def testGetGraphRoUri(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test graph", "ro-testRoGraph")
        rograph = ro_manifest.readManifestGraph(rodir)
        self.assertEquals(ro_manifest.getGraphRoUri(rodir, rograph),
                          rdflib.URIRef("file://%s/RO_test_graph/"%(robase_abs)))
        self.deleteTestRo(rodir)
        return

    def testGetAnnotationByName(self):
        def testAnnotaton(name, expecteduri, expectedtype):
            (apred, atype) = ro_annotation.getAnnotationByName(ro_config, name)
            self.assertEqual(apred, expecteduri)
            self.assertEqual(atype, expectedtype)
            return
        self.assertEqual(RDF.type, rdflib.URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"))
        testAnnotaton("title",          DCTERMS.title,          "string")
        testAnnotaton("description",    DCTERMS.description,    "text")
        testAnnotaton("rdf:type",       RDF.type,               "resource")
        # testAnnotaton(rdflib.URIRef("http://example.org/foo"),  "<http://example.org/foo>")
        return

    def testGetAnnotationByUri(self):
        def testAnnotaton(uri, expectedname, expectedtype):
            (aname, atype) = ro_annotation.getAnnotationByUri(ro_config, uri)
            self.assertEqual(aname, expectedname)
            self.assertEqual(atype, expectedtype)
            return
        testAnnotaton(DCTERMS.title,          "title",          "string")
        testAnnotaton(DCTERMS.description,    "description",    "text")
        testAnnotaton(RDF.type,               "rdf:type",       "resource")
        testAnnotaton(RDFS.comment,           "rdfs:comment",   "string")
        return

    def testGetAnnotationNameByUri(self):
        def testAnnotatonName(uri, name):
            self.assertEqual(ro_annotation.getAnnotationNameByUri(ro_config, uri), name)
            return
        self.assertEqual(RDF.type, rdflib.URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"))
        testAnnotatonName(DCTERMS.title,        "title")
        testAnnotatonName(DCTERMS.description,  "description")
        testAnnotatonName(RDF.type,             "rdf:type")
        testAnnotatonName(RDFS.comment,         "rdfs:comment")
        testAnnotatonName(rdflib.URIRef("http://example.org/foo"),  "<http://example.org/foo>")
        return

    def testMakeAnnotationFilename(self):
        rodir = "/example/ro/dir"
        def testAnnotationFileName(filename, expectedname):
            aname = ro_annotation.makeAnnotationFilename(rodir, filename)
            self.assertEqual(aname, expectedname)
            return
        testAnnotationFileName("a/b", "%s/%s/a/b"%(rodir, ro_settings.MANIFEST_DIR))
        testAnnotationFileName("a/",  "%s/%s/a/"%(rodir, ro_settings.MANIFEST_DIR))
        return

    def testCreateReadRoAnnotationBody(self):
        """
        Test function to create simple annotation body
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        roresource = "."
        attrdict = {
            "type":         "Research Object",
            # @@TODO: handle lists "keywords":     ["test", "research", "object"],
            "description":  "Test research object",
            "format":       "application/vnd.wf4ever.ro",
            "note":         "Research object created for annotation testing",
            "title":        "Test research object",
            "created":      "2011-12-07"
            }
        annotationfilebase = ro_annotation.createAnnotationBody(
            ro_config, rodir, roresource, attrdict)
        # Ann-%04d%02d%02d-%04d-%s.rdf
        self.assertRegexpMatches(annotationfilebase,
            r"Ann-\d\d\d\d\d\d\d\d-\d+-RO_test_annotation\.rdf",
            msg="Unexpected filename form for annotation: "+annotationfilebase)
        annotationfilename = ro_annotation.makeAnnotationFilename(rodir, annotationfilebase)
        annotationgraph    = ro_annotation.readAnnotationBody(rodir, annotationfilename)
        attrpropdict = {
            "type":         DCTERMS.type,
            # @@TODO "keywords":     DCTERMS.subject,
            "description":  DCTERMS.description,
            "format":       DCTERMS.format,
            "note":         ROTERMS.note,
            "title":        DCTERMS.title,
            "created":      DCTERMS.created
            }
        s = ro_manifest.getComponentUri(rodir, roresource)
        log.debug("annotation subject %s"%repr(s))
        for k in attrpropdict:
            p = attrpropdict[k]
            log.debug("annotation predicate %s"%repr(p))
            v = attrdict[k]
            a = annotationgraph.value(s, p, None)
            log.debug("annotation value %s"%repr(a))
            #self.assertEqual(len(a), 1, "Singleton result expected")
            self.assertEqual(a, v)
        self.deleteTestRo(rodir)
        return

    def testCreateReadFileAnnotationBody(self):
        """
        Test function to create simple annotation body
        """
        rodir = self.createTestRo(testbase, "data/ro-test-1", "RO test annotation", "ro-testRoAnnotate")
        roresource = "subdir1/subdir1-file.txt"
        attrdict = {
            "type":         "Test file",
            "description":  "File in test research object",
            "note":         "File in research object created for annotation testing",
            "title":        "Test file in RO",
            "created":      "2011-12-07"
            }
        annotationfilebase = ro_annotation.createAnnotationBody(
            ro_config, rodir, roresource, attrdict)
        # Ann-%04d%02d%02d-%04d-%s.rdf
        self.assertRegexpMatches(annotationfilebase,
            r"Ann-\d\d\d\d\d\d\d\d-\d+-subdir1-file\.txt\.rdf",
            msg="Unexpected filename form for annotation: "+annotationfilebase)
        annotationfilename = ro_annotation.makeAnnotationFilename(rodir, annotationfilebase)
        annotationgraph = ro_annotation.readAnnotationBody(rodir, annotationfilename)
        attrpropdict = {
            "type":         DCTERMS.type,
            "description":  DCTERMS.description,
            "note":         ROTERMS.note,
            "title":        DCTERMS.title,
            "created":      DCTERMS.created
            }
        s = ro_manifest.getComponentUri(rodir, roresource)
        log.debug("annotation subject %s"%repr(s))
        for k in attrpropdict:
            p = attrpropdict[k]
            log.debug("annotation predicate %s"%repr(p))
            v = attrdict[k]
            a = annotationgraph.value(s, p, None)
            log.debug("annotation value %s"%repr(a))
            #self.assertEqual(len(a), 1, "Singleton result expected")
            self.assertEqual(a, v)
        self.deleteTestRo(rodir)
        return

    def testGetInitialRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test init RO annotation", "ro-testRoAnnotate")
        roresource = "."
        # Retrieve the anotations
        annotations = ro_annotation._getRoAnnotations(rodir)
        rouri = ro_manifest.getRoUri(rodir)
        expected_annotations = (
            [ (rouri, DCTERMS.description,  rdflib.Literal('Test init RO annotation'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Test init RO annotation'))
            , (rouri, DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(6+1):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test add RO annotation", "ro-testRoAnnotate")
        roresource = "."
        # Add anotations for RO
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Research Object")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object created for annotation testing")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "Added description")
        # Retrieve the anotations
        annotations = ro_annotation._getRoAnnotations(rodir)
        rouri = ro_manifest.getRoUri(rodir)
        expected_annotations = (
            [ (rouri, DCTERMS.description,  rdflib.Literal('Test add RO annotation'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Test add RO annotation'))
            , (rouri, DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            , (rouri, DCTERMS.type,         rdflib.Literal('Research Object'))
            , (rouri, ROTERMS.note,         rdflib.Literal('Research object created for annotation testing'))
            , (rouri, DCTERMS.description,  rdflib.Literal('Added description'))
            ])
        for i in range(9+4):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testRemoveGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test remove RO annotation", "ro-testRoAnnotate")
        roresource = "."
        # Remove some anotations for RO
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Research Object")
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "title",        "Test remove RO annotation")
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "description",  None)
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object created for annotation testing")
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "created",      None)
        # Retrieve the anotations
        annotations = ro_annotation._getRoAnnotations(rodir)
        rouri = ro_manifest.getRoUri(rodir)
        expected_annotations = (
            [ (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(4):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testReplaceGetRoAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test replace RO annotation", "ro-testRoAnnotate")
        roresource = "."
        # Replace anotations for RO
        ro_annotation._replaceSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Research Object")
        ro_annotation._replaceSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "Replacement description")
        ro_annotation._replaceSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object for annotation replacement testing")
        ro_annotation._replaceSimpleAnnotation(ro_config, rodir, roresource,
            "title",        "Replacement title")
        ro_annotation._replaceSimpleAnnotation(ro_config, rodir, roresource,
            "created",      "2011-12-07")
        # Retrieve the anotations
        annotations = ro_annotation._getRoAnnotations(rodir)
        rouri = ro_manifest.getRoUri(rodir)
        expected_annotations = (
            [ (rouri, DCTERMS.type,         rdflib.Literal('Research Object'))
            , (rouri, DCTERMS.title,        rdflib.Literal('Replacement title'))
            , (rouri, DCTERMS.description,  rdflib.Literal('Replacement description'))
            , (rouri, ROTERMS.note,         rdflib.Literal('Research object for annotation replacement testing'))
            , (rouri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (rouri, DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri, DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri, RDF.type,             RO.ResearchObject)
            ])
        for i in range(8+1):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetFileAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test add file annotation", "ro-testRoAnnotate")
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Test file")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "File in test research object")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object file created for annotation testing")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "title",        "Test file in RO")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "created",      "2011-12-07")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "rdf:type",     ROTERMS.resource)
        # Retrieve the file anotations
        annotations = ro_annotation._getFileAnnotations(rodir, roresource)
        resourceuri = ro_manifest.getComponentUri(rodir, roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.description,  rdflib.Literal('File in test research object'))
            , (resourceuri, ROTERMS.note,         rdflib.Literal('Research object file created for annotation testing'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(6):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testRemoveGetFileAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test remove file annotation", "ro-testRoAnnotate")
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Test file")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "File in test research object")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object file created for annotation testing")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "title",        "Test file in RO")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "created",      "2011-12-07")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "rdf:type",     ROTERMS.resource)
        # Remove annotations
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "File in test research object")
        ro_annotation._removeSimpleAnnotation(ro_config, rodir, roresource,
            "note",         None)
        # Retrieve the file anotations
        annotations = ro_annotation._getFileAnnotations(rodir, roresource)
        resourceuri = ro_manifest.getComponentUri(rodir, roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(4):
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetAllAnnotations(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test get all annotations", "ro-testRoAnnotate")
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Test file")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "File in test research object")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "note",         "Research object file created for annotation testing")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "title",        "Test file in RO")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "created",      "2011-12-07")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "rdf:type",     ROTERMS.resource)
        # Retrieve the file anotations
        annotations = ro_annotation.getAllAnnotations(rodir)
        rouri       = ro_manifest.getRoUri(rodir)
        resourceuri = ro_manifest.getComponentUri(rodir, roresource)
        log.debug("resourceuri: %s"%(resourceuri))
        expected_annotations = (
            [ (rouri,       DCTERMS.description,  rdflib.Literal('Test get all annotations'))
            , (rouri,       DCTERMS.title,        rdflib.Literal('Test get all annotations'))
            , (rouri,       DCTERMS.created,      rdflib.Literal('unknown'))
            , (rouri,       DCTERMS.creator,      rdflib.Literal('Test User'))
            , (rouri,       DCTERMS.identifier,   rdflib.Literal('ro-testRoAnnotate'))
            , (rouri,       RDF.type,             RO.ResearchObject)
            , (resourceuri, DCTERMS.type,         rdflib.Literal('Test file'))
            , (resourceuri, DCTERMS.description,  rdflib.Literal('File in test research object'))
            , (resourceuri, ROTERMS.note,         rdflib.Literal('Research object file created for annotation testing'))
            , (resourceuri, DCTERMS.title,        rdflib.Literal('Test file in RO'))
            , (resourceuri, DCTERMS.created,      rdflib.Literal('2011-12-07'))
            , (resourceuri, RDF.type,             ROTERMS.resource)
            ])
        for i in range(12+1+6):      # Annotations + aggregations
            next = annotations.next()
            #log.debug("Next %s"%(repr(next)))
            if ( next not in expected_annotations and
                 next[1] != DCTERMS.created       and
                 next[1] != ORE.aggregates        ):
                self.assertTrue(False, "Not expected (%d) %s"%(i, repr(next)))
        self.assertRaises(StopIteration, annotations.next)
        self.deleteTestRo(rodir)
        return

    def testAddGetAnnotationValues(self):
        rodir = self.createTestRo(testbase, "data/ro-test-1", "Test get annotation values", "ro-testRoAnnotate")
        roresource = "subdir1/subdir1-file.txt"
        # Add anotations for file
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "type",         "Test file")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "description",  "File in test research object")
        ro_annotation._addSimpleAnnotation(ro_config, rodir, roresource,
            "rdf:type",     ROTERMS.resource)
        # Retrieve the anotations
        values = ro_annotation._getAnnotationValues(ro_config, rodir, ".", "title")
        self.assertEquals(values.next(), rdflib.Literal('Test get annotation values'))
        self.assertRaises(StopIteration, values.next)
        values = ro_annotation._getAnnotationValues(ro_config, rodir, ".", "rdf:type")
        self.assertEquals(values.next(), RO.ResearchObject)
        self.assertRaises(StopIteration, values.next)
        values = ro_annotation._getAnnotationValues(ro_config, rodir, roresource, "type")
        self.assertEquals(values.next(), rdflib.Literal('Test file'))
        self.assertRaises(StopIteration, values.next)
        values = ro_annotation._getAnnotationValues(ro_config, rodir, roresource, "description")
        self.assertEquals(values.next(), rdflib.Literal('File in test research object'))
        self.assertRaises(StopIteration, values.next)
        values = ro_annotation._getAnnotationValues(ro_config, rodir, roresource, "rdf:type")
        self.assertEquals(values.next(), ROTERMS.resource)
        self.assertRaises(StopIteration, values.next)
        # Clean up
        self.deleteTestRo(rodir)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testGetFileUri"
            , "testGetUriFile"
            , "testGetRoUri"
            , "testGetComponentUri"
            , "testGetComponentUriRel"
            , "testGetGraphRoUri"
            , "testGetAnnotationByName"
            , "testGetAnnotationByUri"
            , "testGetAnnotationNameByUri"
            , "testMakeAnnotationFilename"
            , "testCreateReadRoAnnotationBody"
            , "testCreateReadFileAnnotationBody"
            , "testGetInitialRoAnnotations"
            , "testAddGetRoAnnotations"
            , "testRemoveGetRoAnnotations"
            , "testReplaceGetRoAnnotations"
            , "testAddGetFileAnnotations"
            , "testRemoveGetFileAnnotations"
            , "testAddGetAllAnnotations"
            , "testAddGetAnnotationValues"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestAnnotationUtils, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestAnnotationUtils.log", getTestSuite, sys.argv)

# End.



# ro_manifest.py

"""
Research Object manifest read, write, decode functions
"""

import sys
import os
import os.path
import re
import urlparse
import logging

log = logging.getLogger(__name__)

import MiscLib.ScanDirectories

import rdflib
###import rdflib.namespace
#from rdflib import URIRef, Namespace, BNode
#from rdflib import Literal

import ro_settings
from ro_namespaces import RDF, DCTERMS, RO, AO, ORE

def makeManifestFilename(rodir):
    return os.path.join(rodir, ro_settings.MANIFEST_DIR+"/", ro_settings.MANIFEST_FILE)

def readManifestGraph(rodir):
    """
    Read manifest file for research object, return RDF Graph of manifest values.
    """
    manifestfilename = makeManifestFilename(rodir)
    rdfGraph = rdflib.Graph()
    rdfGraph.parse(manifestfilename)
    return rdfGraph

def writeManifestGraph(rodir, rograph):
    """
    Write manifest file for research object given RDF graph of contents
    """
    manifestfilename = makeManifestFilename(rodir)
    rograph.serialize(destination=manifestfilename, format='xml', base=getRoUri(rodir), xml_base="..")
    return

def readManifest(rodir):
    """
    Read manifest file for research object, return dictionary of manifest values.
    """
    rdfGraph = readManifestGraph(rodir)
    subject  = rdfGraph.value(None, RDF.type, RO.ResearchObject)
    strsubject = ""
    if isinstance(subject, rdflib.URIRef): strsubject = str(subject)
    manifestDict = {
        'ropath':         rodir,
        'rouri':          strsubject,
        'roident':        rdfGraph.value(subject, DCTERMS.identifier,  None),
        'rotitle':        rdfGraph.value(subject, DCTERMS.title,       None),
        'rocreator':      rdfGraph.value(subject, DCTERMS.creator,     None),
        'rocreated':      rdfGraph.value(subject, DCTERMS.created,     None),
        'rodescription':  rdfGraph.value(subject, DCTERMS.description, None),
        }
    return manifestDict

def notHidden(f):
    return re.match("\.|.*/\.", f) == None

def addAggregatedResources(ro_dir, ro_file, recurse=True):
    log.debug("addAggregatedResources: dir %s, file %s"%(ro_dir, ro_file))
    ro_graph = readManifestGraph(ro_dir)
    if ro_file.endswith(os.path.sep):
        ro_file = ro_file[0:-1]
    rofiles = [ro_file]
    if os.path.isdir(ro_file):
        rofiles = filter( notHidden,
                            MiscLib.ScanDirectories.CollectDirectoryContents(
                                os.path.abspath(ro_file), baseDir=os.path.abspath(ro_dir), 
                                listDirs=False, listFiles=True, recursive=recurse, appendSep=False)
                        )
    s = getComponentUri(ro_dir, ".")
    for f in rofiles:
        log.debug("- file %s"%f)
        stmt = (s, ORE.aggregates, getComponentUri(ro_dir, f))
        if stmt not in ro_graph: ro_graph.add(stmt)
    writeManifestGraph(ro_dir, ro_graph)
    return

def getAggregatedResources(ro_dir):
    """
    Returns iterator over all resources aggregated by a manifest.
    
    Each value returned by the iterator is a resource URI.
    """
    ro_graph = readManifestGraph(ro_dir)
    subject  = getRoUri(ro_dir)
    log.debug("getAggregatedResources %s"%str(subject))
    for r in ro_graph.objects(subject=subject, predicate=ORE.aggregates):
        yield r
    return

def getFileUri(path):
    """
    Like getComponentUri, except that path may be relative to the current directory
    """
    filebase = "file://"
    if not path.startswith(filebase):
        path = filebase+os.path.join(os.getcwd(), path)
    return rdflib.URIRef(path)

def getUriFile(uri):
    """
    Return file path string corresponding to supplied RO or RO component URI
    """
    filebase = "file://"
    uri = str(uri)
    if uri.startswith(filebase):
        uri = uri[len(filebase):]
    return uri

def old_getRoUri(ro_dir):
    return getFileUri(os.path.abspath(ro_dir)+"/")

def getRoUri(roref):
    base = "file://"+os.path.abspath(os.getcwd())+"/"
    uri  = urlparse.urljoin(base, roref)
    if not uri.endswith("/"): uri += "/" 
    return rdflib.URIRef(uri)

def getComponentUri(ro_dir, path):
    #log.debug("getComponentUri: ro_dir %s, path %s"%(ro_dir, path))
    return rdflib.URIRef(urlparse.urljoin(str(getRoUri(ro_dir)), path))
    #return rdflib.URIRef("file://"+os.path.normpath(os.path.join(os.path.abspath(ro_dir), path)))

def getComponentUriRel(ro_dir, path):
    #log.debug("getComponentUriRel: ro_dir %s, path %s"%(ro_dir, path))
    file_uri = urlparse.urlunsplit(urlparse.urlsplit(str(getComponentUri(ro_dir, path))))
    ro_uri   = urlparse.urlunsplit(urlparse.urlsplit(str(getRoUri(ro_dir))))
    #log.debug("getComponentUriRel: ro_uri %s, file_uri %s"%(ro_uri, file_uri))
    if ro_uri is not None and file_uri.startswith(ro_uri):
        file_uri_rel = file_uri.replace(ro_uri, "", 1)
    else:
        file_uri_rel = path
    #log.debug("getComponentUriRel: file_uri_rel %s"%(file_uri_rel))
    return rdflib.URIRef(file_uri_rel)

def getGraphRoUri(rodir, rograph):
    """
    Extract graph URI from supplied manifest graph
    """
    return rograph.value(None, RDF.type, RO.ResearchObject)

# End.



'''
Created on 15-09-2011

@author: piotrhol
'''
import sys
if __name__ == "__main__":
    # Add main project directory and ro manager directories at start of python path
    sys.path.insert(0, "../..")
    sys.path.insert(0, "..")

import logging
import random
import string
import os.path
from MiscLib import TestUtils
from rocommand.test import TestROSupport
from rocommand.test.TestConfig import ro_test_config
from sync.RosrsApi import RosrsApi
from sync.ResourceSync import ResourceSync

# Logging object
log = logging.getLogger(__name__)

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

class TestResourceSync(TestROSupport.TestROSupport):
    
    files = ["subdir1/subdir1-file.txt"
             , "subdir2/subdir2-file.txt"
             , "README-ro-test-1"
             , ".ro/manifest.rdf"
             , "minim.rdf" ]

    fileToModify = "subdir1/subdir1-file.txt"
    fileToDelete = "README-ro-test-1"
    fileToReplace = "README-ro-test-2"
    fileToTouch = 'subdir2/subdir2-file.txt'
    
    modifiedFileContent = """lorem ipsum
ora et labora"""

    def setUp(self):
        super(TestResourceSync, self).setUp()
        self.setupConfig()
        self.ident1 = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(16))
        self.rodir = self.createTestRo(testbase, "../../rocommand/test/data/ro-test-1", self.ident1, self.ident1)
        self.workspacedir = os.path.join(testbase, ro_test_config.ROBASEDIR)
        try:
            self.__sync = RosrsApi(ro_test_config.ROSRS_URI, ro_test_config.ROSRS_ACCESS_TOKEN)
        except:
            pass
        logging.basicConfig()
        logging.getLogger("sync.ResourceSync").setLevel(logging.INFO)
        
        tmpFiles = set()
        for f in self.files:
            tmpFiles.add(os.path.join(self.rodir, f))
        self.files = tmpFiles
        self.fileToModify = os.path.join(self.rodir, self.fileToModify)
        self.fileToDelete = os.path.join(self.rodir, self.fileToDelete)
        self.fileToReplace = os.path.join(self.rodir, self.fileToReplace)
        self.fileToTouch = os.path.join(self.rodir, self.fileToTouch)
        return

    def tearDown(self):
        super(TestResourceSync, self).tearDown()
        self.deleteTestRo(self.rodir)
        try:
            self.__sync.deleteRo(self.ident1)
        except:
            pass
        return
    
    # Setup local config for local tests

    def setupConfig(self):
        return self.setupTestBaseConfig(testbase)

    def testNull(self):
        assert True, 'Null test failed'

    def testSyncRecources(self):
        self.__sync.postRo(self.ident1)
        back = ResourceSync(self.__sync)
        
        (sent, deleted) = back.pushAllResources(self.rodir, force = True)
        self.assertEquals(sent, self.files, "Sent files are not equal")
        self.assertEquals(deleted, set())

        os.rename(self.fileToDelete, self.fileToReplace)
        os.utime(self.fileToTouch, None)
        with open(self.fileToModify, 'a') as f:
            f.write("foobar")
            f.close()
        
        (sent, deleted) = back.pushAllResources(self.rodir)
        self.assertEquals(sent, {self.fileToReplace, self.fileToModify}, "New sent file")
        self.assertEquals(deleted, {self.fileToDelete}, "Deleted file")

        (sent, deleted) = back.pushAllResources(self.rodir)
        self.assertEquals(sent, set())
        self.assertEquals(deleted, set())
        os.rename(self.fileToReplace, self.fileToDelete)
        return
    
    def testSyncWorkspace(self):
        back = ResourceSync(self.__sync)
        self.assertRaises(Exception, back.pushAllResourcesInWorkspace, self.workspacedir, force = True)
        back = ResourceSync(self.__sync)
        (sent, deleted) = back.pushAllResourcesInWorkspace(self.workspacedir, True, True)
        log.debug("testSyncWorkspace: files: [%s]"%(", ".join(self.files)))
        log.debug("testSyncWorkspace: sent: [%s]"%(", ".join(sent)))
        self.assertSetEqual(sent, self.files, "Send all workspace resource")
        self.assertSetEqual(deleted, set())
        self.assertTupleEqual((set(), set()), back.pushAllResourcesInWorkspace(self.workspacedir), 
                              "Sync workspace after creating RO")
        return
    
    def testSaveLoadRegistries(self):
        back = ResourceSync(self.__sync)
        (sent, deleted) = back.pushAllResourcesInWorkspace(self.workspacedir, True, True)
        self.assertEquals(sent, self.files, "Send all workspace resource")
        self.assertEquals(deleted, set())
        back = ResourceSync(self.__sync)
        self.assertTupleEqual((set(), set()), back.pushAllResourcesInWorkspace(self.workspacedir), 
                              "Sync workspace after loading registries")
        back = ResourceSync(self.__sync)
        (sent, deleted) = back.pushAllResourcesInWorkspace(self.workspacedir, True, True)
        self.assertEquals(sent, self.files, "Send all workspace resource")
        self.assertEquals(deleted, set())
        return
    
def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            ],
        "component":
            [ "testComponents"
            , "testSyncRecources"
            , "testSyncWorkspace"
            , "testSaveLoadRegistries"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestResourceSync, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestResourceSync.log", getTestSuite, sys.argv)



'''
Created on 15-09-2011

@author: piotrhol
'''

from time import time
from os import path
import hashlib

class SyncRegistry:
    '''
    classdocs
    '''
    
    filename = None
    lastSyncTime = None
    checksum = None


    def __init__(self, filename, time = time(), checksum = None):
        '''
        Constructor
        '''
        self.filename = filename
        self.lastSyncTime = time
        self.checksum = self.__calculateMd5()
        
    def hasBeenModified(self):
        if (path.getmtime(self.filename) > self.lastSyncTime):
            if (self.__calculateMd5() != self.checksum):
                return True
        return False
        
    def __calculateMd5(self):
        m = hashlib.md5()
        with open(self.filename) as f:
            for line in f:
                m.update(line)
            f.close()
        return m.hexdigest()
        
    


#!/usr/bin/env python

"""
Module to test RO SRS APIfunctions
"""

import os, os.path
import sys
import unittest
import logging
import json
import StringIO
import httplib
#import urllib
import urlparse
import rdflib

from MiscLib import TestUtils

# Logging object
log = logging.getLogger(__name__)

# Class for HTTP session errors

class HTTPSessionError(Exception):

    def __init__(self, msg="HTTPSessionError", value=None, uri=None):
        self._msg    = msg
        self._value  = value
        self._uri = uri
        return

    def __str__(self):
        str = self._msg
        if self._uri:   str += " for "+self._uri
        if self._value: str += ": "+repr(value)
        return str

    def __repr__(self):
        return ( "HTTPSessionError(%s, value=%s, uri=%s)"%
                 (repr(self._msg), repr(self._value), repr(self._uri)))

# Class for handling HTTP access

class HTTP_Session(object):

    def __init__(self, uri, accesskey=None):
        self._uri     = uri
        self._key     = accesskey
        parseduri     = urlparse.urlsplit(uri)
        self._scheme  = parseduri.scheme
        self._host    = parseduri.netloc
        self._path    = parseduri.path
        self._httpcon = httplib.HTTPConnection(self._host)
        return

    def close(self):
        self._key = None
        self._httpcon.close()
        return

    def baseuri(self):
        return self._uri

    def error(self, msg, value):
        return HTTPSessionError(msg=msg, value=value, uri=self._uri)

    def parseLinks(headers):
        """
        Parse link header(s), return dictionary of links keyed by link relation type
        """
        linkvallist = [ v for (h,v) in headers["_headerlist"] if h == "link" ]
        links = {}
        for linkval in linkvallist:
            linkmatch = re.match(r'''\s*<([^>]*)>\s*;\s*rel\s*=\s*"([^"]*)"''', linkval)
            if linkmatch:
                links[linkmatch.group(2)] = rdflib.URIRef(linkmatch.group(1))
        return links

    def doRequest(self, uripath, method="GET", body=None, ctype=None, accept=None, reqheaders=None):
        """
        Perform HTTP request
        Return status, reason(text), response headers, response body
        """
        # Sort out path to use in HTTP request: request may be path or full URI or rdflib.URIRef
        uripath  = str(uripath)        # get URI string from rdflib.URIRef
        uriparts = urlparse.urlsplit(urlparse.urljoin(self._path,uripath))
        if uriparts.scheme:
            if self._scheme != uriparts.scheme:
                raise HTTPSessionError(
                    "HTTP session URI scheme mismatch",
                    value=uriparts.scheme,
                    uri=self._uri)
        if uriparts.netloc:
            if self._host != uriparts.netloc:
                raise HTTPSessionError(
                    "HTTP session URI host:port mismatch",
                    value=uriparts.netloc,
                    uri=self._uri)
        path = uriparts.path
        if uriparts.query: path += "?"+uriparts.query
        # Assemble request headers
        if not reqheaders:
            reqheaders = {}
        if self._key:
            reqheaders["authorization"] = "Bearer "+self._key
        if ctype:
            reqheaders["content-type"] = ctype
        if accept:
            reqheaders["accept"] = accept
        # Execute request
        self._httpcon.request(method, path, body, reqheaders)
        # Pick out elements of response
        response = self._httpcon.getresponse()
        status   = response.status
        reason   = response.reason
        headerlist = [ (h.lower(),v) for (h,v) in response.getheaders() ]
        headers  = dict(headerlist)   # dict(...) keeps last result of multiple keys
        headers["_headerlist"] = headerlist
        data = response.read()
        return (status, reason, headers, data)

    def doRequestRDF(self, uripath, method="GET", body=None, ctype=None, headers=None):
        """
        Perform HTTP request with RDF response.
        If requests succeeds, return response as RDF graph,
        or return fake 9xx status if RDF cannot be parsed
        otherwise return responbse and content per request.
        Thus, only 2xx responses include RDF data.
        """
        (status, reason, headers, data) = self.doRequest(uripath,
            method=method, body=body,
            ctype=ctype, accept="application/rdf+xml", headers=headers)
        if status >= 200 and status < 300:
            if headers["content-type"].lower() == "application/rdf+xml":
                rdfgraph = rdflib.Graph()
                try:
                    rdfgraph.parse(data=data, format="xml")
                    data = rdfgraph
                except Exception, e:
                    status   = 902
                    reason   = "RDF parse failure"
            else:
                status   = 901
                reason   = "Non-RDF content-type returned"
        return (status, reason, headers, data)

# End.



# $Id: SuperGlobal.py 1047 2009-01-15 14:48:58Z graham $

import __main__

class SuperGlobal:
    """
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/457667

    Creates globals.

    i.e.
    superglobal = SuperGlobal()
    superglobal.data = ....

    However many times you create SuperGlobal it access the same data.
    """

    def __getattr__(self, name):
        return __main__.__dict__.get(name, None)
        
    def __setattr__(self, name, value):
        __main__.__dict__[name] = value
        
    def __delattr__(self, name):
        if __main__.__dict__.has_key(name):
            del  __main__.__dict__[name]



# $Id: NetUtils.py 1047 2009-01-15 14:48:58Z graham $
#
#   Network utilities
#
import logging
from string import split

from Functions  import all, formatIntList, formatInt

def ipAdrStrToInt( adrStr ):
    """
    Convert a dotted ip address to 32 bit integer.
    """
    adrParts = split( adrStr, ".", 3 )
    return (int(adrParts[0]) << 24) + (int(adrParts[1]) << 16) + (int(adrParts[2]) << 8) + int(adrParts[3]) 

def addBroadcastBits( iAdr, bitCount ):
    """
    iAdr is 32 bit integer 
    bitCount is integer.
    """
    # set the broadcast values
    for idx in range( 32-bitCount ):
        iAdr = iAdr | (1 << idx)
    return iAdr

def getBroadcastAddressI( adrStr, bitStr ):
    """
    Returns address as Integer
    """
    # else has netmask part.
    iAdr = ipAdrStrToInt( adrStr ) # integer address
    bAdr = addBroadcastBits( iAdr, int( bitStr ) )
    return bAdr

def getBroadcastAddress( adrStr ):
    """
    Convert an ip address in form nn.nn.nn.nn/bb into its broadcast address format.
    b/bb is optional and assumes caller knows what they are doing.
    """
    netParts = split( adrStr, "/", 1 )
    if ( len(netParts) == 1 ):
        return adrStr
    # else has netmask part.
    # else has netmask part.
    iAdr = ipAdrStrToInt( netParts[0] ) # integer address
    bAdr = getBroadcastAddressI( netParts[0], netParts[1] )
    return "%i.%i.%i.%i" % ( ((bAdr>>24)&0xFF), ((bAdr>>16)&0xFF), ((bAdr>>8)&0xFF), (bAdr&0xFF) )


# Helper functions for processing IP addresses as lists of int values
# (I think this representation will be easier to migrate to also support IPv6 - GK)

def parseIpAdrs(ipadrs):
    """
    Parse IP address in dotted decomal form, and return a sequence of 4 numbers
    """
    # Strip of any port and/or netmask bits
    ipadrs  = ipadrs.split('/')[0].split(':')[0]
    return map(int, ipadrs.split('.'))

def parseNetAdrs(netadrs):
    """
    Parse network address specification, returning a pair of:
    (a) IP address bytes tuple
    (b) number of '1' bits in netmask
    """
    (ipadrs,maskbits) = netadrs.split('/')
    return (parseIpAdrs(ipadrs),int(maskbits))

def formatIpAdrs(ipbytes):
    """
    Format IP address string from IP addre4ss bytes
    """
    # return "%d.%d.%d.%d" % ipbytes
    return formatIntList(ipbytes,".")

def formatNetAdrs(ipbytes,maskbits):
    """
    Format network address string from IP address bytes and mask bit count
    """
    return formatIpAdrs(ipbytes)+("/%d" % maskbits)

def mkNetMask(ipbytes,maskbits=None):
    """
    Make a network mask value as a sequence of IP address bytes
    
    May be called with 1 or 2 arguments:  
    if 1 argument,  it is a pair of (netbytes,maskbits)
    if 2 arguments, the first is just netbytes, and the second is maskbits
    """
    if not maskbits: (ipbytes,maskbits) = ipbytes
    netmask = []
    for b in ipbytes:
        m = 0
        if   maskbits >= 8: 
            m = 255
        elif maskbits >  0: 
            m = (0,
                 128,
                 128+64,
                 128+64+32,
                 128+64+32+16,
                 128+64+32+16+8,
                 128+64+32+16+8+4,
                 128+64+32+16+8+4+2)[maskbits]
        netmask.append(m)
        maskbits -= 8
    return netmask

def mkBroadcastAddress(netbytes,maskbits=None):
    """
    Make broadcast address for a given network
    
    May be called with 1 or 2 arguments:  
    if 1 argument,  it is a pair of (netbytes,maskbits)
    if 2 arguments, the first is just netbytes, and the secvond is maskbits
    """
    def makeadrbyte(m, a): return (~m | a) & 0xFF
    if not maskbits: (netbytes,maskbits) = netbytes
    netmask = mkNetMask(netbytes,maskbits)
    return map(makeadrbyte, netmask, netbytes)

def ipInNetwork(ipbytes, netbytes, maskbits=None):
    """
    Test if IP address is part of given network
    
    May be called with 2 or 3 arguments:  
    if 2 arguments, the second is a pair of (netbytes,maskbits)
    if 3 arguments, the second is just netbytes, and the third is maskbits
    """
    def testadrbyte(m, n, a): return (m & a) == (m & n)
    if not maskbits: (netbytes,maskbits) = netbytes
    netmask = mkNetMask(netbytes, maskbits)
    return all(testadrbyte, netmask, netbytes, ipbytes)

def getHostIpsAndMask():
    """
    Helper function returns list of IP networks connected to the
    current host.
    
    Each value is in the form address/maskbits, e.g.
      10.0.0.0/8
    """
    result = list()
    from socket import gethostbyname_ex, gethostname
    try:
        hosts = gethostbyname_ex( gethostname( ) )
        for addr in hosts[2]:
            # convert to ...
            byts = parseIpAdrs(addr)
            if byts[0] >= 192:
                # class C
                result.append( "%i.%i.%i.0/24" % (byts[0],byts[1],byts[2]) )
            elif byts[0] >= 128:
                # class B
                result.append( "%i.%i.0.0/16" % (byts[0],byts[1]) )
            else:
                # class A
                result.append( "%i.0.0.0/8" % (byts[0]) )
    except Exception, ex :
        _log = logging.getLogger('WebBrickLibs.MiscLib.NetUtils')
        _log.exception(ex)
    return result

# Helper functions for processing MAC addresses as lists of integers

def parseMacAdrs(macadrs):
    """
    Parse Mac address in colon-hexadecimal form, and return a sequence of 6 numbers
    """
    def hex(h): return int(h,16)
    return map(hex, macadrs.split(':'))

def formatMacAdrs(macbytes,sep=":"):
    """
    Format MAC address as colon-separated hexadecimals for webBrick command
    """
    return formatIntList(macbytes, sep, formatInt("%02X"))

# test cases
def _test():
    i = parseIpAdrs("193.123.216.121")
    x = parseIpAdrs("193.123.216.200")
    n = parseNetAdrs("193.123.216.64/26")
    b = mkBroadcastAddress(*n)
    assert formatIpAdrs(b) == "193.123.216.127"
    assert ipInNetwork(i,*n)
    assert ipInNetwork(b,*n)
    assert not ipInNetwork(x,*n)
    assert parseMacAdrs("01:34:67:9a:BC:eF") == [1,52,103,154,188,239]
    assert formatMacAdrs([1,52,103,154,188,239],sep='-') == "01-34-67-9A-BC-EF"

_test()

# End  $Id: NetUtils.py 1047 2009-01-15 14:48:58Z graham $



#!/usr/bin/env python
#
# Setup.py based on https://github.com/paltman/python-setup-template/blob/master/setup.py,
# following http://www.ibm.com/developerworks/opensource/library/os-pythonpackaging/index.html

import codecs
import os
import sys

from distutils.util import convert_path
from fnmatch import fnmatchcase
from setuptools import setup, find_packages

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

# (Leftover from previous setup?)
standard_exclude = ["*.py", "*.pyc", "*$py.class", "*~", ".*", "*.bak"]
standard_exclude_directories = [
    ".*", "CVS", "_darcs", "./build", "./dist", "EGG-INFO", "*.egg-info"
]

PACKAGE = "rocommand"
NAME = "ro-manager"
DESCRIPTION = "Research Object manager (command line tool)"
AUTHOR = "Graham Klyne"
AUTHOR_EMAIL = "gk-pypi@ninebynine.org"
URL = "https://github.com/wf4ever/ro-manager"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=read("README.txt"),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="MIT",
    url=URL,
    packages = ['rocommand','rocommand.test'
               ,'iaeval','iaeval.test'
               ,'sync','sync.test'
               ,'MiscLib'
               ,'samples'
               ,'uritemplate'], #@@TODO: uritemplate already in PyPI?
    package_data = {
        'rocommand':    ['test/config/*','test/robase/*','test/nobase/*'
                        ,'test/data/ro-test-1/subdir1/*','test/data/ro-test-1/subdir2/*'
                        ,'test/data/ro-test-1/*.rdf','test/data/ro-test-1/README*'
                        ],
        'iaeval':       ['test/config/*','test/robase/*'
                        ,'test/data/data/*','test/data/docs/*'
                        ,'test/data/*.rdf'
                        ],
        'sync':         ['test/config/*','test/robase/*'],
        'samples':      ['*.sh', '*.py', 'prefixes.*', 'README'],
        },
    exclude_package_data = {
        '': ['spike/*'] 
        },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        ],
    zip_safe=False,
    install_requires=["rdflib >= 3.2.1", "rdfextras >=0.2"],
    entry_points = {
        'console_scripts': [
            'ro = rocommand.ro:runMain',
            'ro-manager-test = rocommand.test.RunRoManagerTests:runTestSuite'
            ],
        },
    )



#!/usr/bin/python

"""
Module to test RO manager manifest and aggregation commands

See: http://www.wf4ever-project.org/wiki/display/docs/RO+management+tool
"""

import os, os.path
import sys
import re
import shutil
import unittest
import logging
import datetime
import StringIO
try:
    # Running Python 2.5 with simplejson?
    import simplejson as json
except ImportError:
    import json

log = logging.getLogger(__name__)

if __name__ == "__main__":
    # Add main project directory at start of python path
    sys.path.insert(0, "../..")

import rdflib

from MiscLib import TestUtils

from rocommand import ro
from rocommand import ro_utils
from rocommand.ro_namespaces import RDF, DCTERMS, RO, AO, ORE
from rocommand.ro_annotation import annotationTypes
from rocommand.ro_metadata   import ro_metadata

from rocommand.test import TestROSupport
from rocommand.test import TestConfig
from rocommand.test import StdoutContext

#from TestConfig import ro_test_config
#from StdoutContext import SwitchStdout

from iaeval import ro_minim
from iaeval.ro_minim import MINIM

from iaeval import ro_eval_minim

# Local ro_config for testing
ro_config = {
    "annotationTypes": annotationTypes
    }

# Base directory for RO tests in this module
testbase = os.path.dirname(os.path.abspath(__file__))

class TestEvalChecklist(TestROSupport.TestROSupport):
    """
    Test ro checklist evaluation

    Some tests here require that the python package lpod-python is installed
      apt-get  install libxml2-dev
      apt-get  install libxslt-dev
      easy_install lxml
      pip install lpod-python
    """
    def setUp(self):
        super(TestEvalChecklist, self).setUp()
        return

    def tearDown(self):
        super(TestEvalChecklist, self).tearDown()
        return

    # Setup local config for Minim tests

    def setupConfig(self):
        return self.setupTestBaseConfig(testbase)

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testSetupConfig(self):
        (configdir, robasedir) = self.setupConfig()
        config = ro_utils.readconfig(configdir)
        self.assertEqual(config["robase"],          os.path.abspath(robasedir))
        self.assertEqual(config["rosrs_uri"],       TestConfig.ro_test_config.ROSRS_URI)
        self.assertEqual(config["rosrs_access_token"],  TestConfig.ro_test_config.ROSRS_ACCESS_TOKEN)
        self.assertEqual(config["username"],        TestConfig.ro_test_config.ROBOXUSERNAME)
        self.assertEqual(config["useremail"],       TestConfig.ro_test_config.ROBOXEMAIL)
        return

    def testEvalAllPresent(self):
        """
        Evaluate complete RO against Minim description 
        """
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        self.populateTestRo(testbase, rodir)
        rometa = ro_metadata(ro_config, rodir)
        (g,evalresult) = ro_eval_minim.evaluate(rometa,
            "Minim-UserRequirements.rdf",               # Minim file
            "docs/UserRequirements-astro.csv",          # Target resource
            "create")                                   # Purpose
        self.assertIn(MINIM.fullySatisfies,     evalresult['summary'])
        self.assertIn(MINIM.nominallySatisfies, evalresult['summary'])
        self.assertIn(MINIM.minimallySatisfies, evalresult['summary'])
        self.assertEquals(evalresult['missingMust'],    [])
        self.assertEquals(evalresult['missingShould'],  [])
        self.assertEquals(evalresult['missingMay'],     [])
        self.assertEquals(evalresult['rouri'],          rometa.getRoUri())
        self.assertEquals(evalresult['minimuri'],       rometa.getComponentUri("Minim-UserRequirements.rdf"))
        self.assertEquals(evalresult['target'],         "docs/UserRequirements-astro.csv")
        self.assertEquals(evalresult['purpose'],        "create")
        self.assertEquals(evalresult['constrainturi'],  
            rometa.getComponentUri("Minim-UserRequirements.rdf#create/docs/UserRequirements-astro.csv"))
        self.assertEquals(evalresult['modeluri'],
            rometa.getComponentUri("Minim-UserRequirements.rdf#runnableRequirementRO"))
        self.deleteTestRo(rodir)
        return

    def testEvalMustMissing(self):
        """
        Evaluate complete RO against Minim description 
        """
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        self.populateTestRo(testbase, rodir)
        rometa = ro_metadata(ro_config, rodir)
        minimbase  = rometa.getComponentUri("Minim-UserRequirements.rdf")
        modeluri   = ro_minim.getElementUri(minimbase, "#missingMustRequirement")
        (g,evalresult) = ro_eval_minim.evaluate(rometa,
            "Minim-UserRequirements.rdf",               # Minim file
            "docs/UserRequirements-bio.csv",            # Target resource
            "create")                                   # Purpose
        missing_must = (
            { 'level': "MUST"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates data/UserRequirements-bio.ods")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("data/UserRequirements-bio.ods")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-bio.ods")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-bio.ods") 
            })
        self.maxDiff=None
        self.assertEquals(evalresult['summary'],       [] )
        self.assertEquals(evalresult['missingMust'],   [(missing_must,{})] )
        self.assertEquals(evalresult['missingShould'], [] )
        self.assertEquals(evalresult['missingMay'],    [] )
        self.deleteTestRo(rodir)
        return

    def testEvalShouldMissing(self):
        """
        Evaluate complete RO against Minim description 
        """
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        self.populateTestRo(testbase, rodir)
        rometa = ro_metadata(ro_config, rodir)
        minimbase  = rometa.getComponentUri("Minim-UserRequirements.rdf")
        modeluri   = ro_minim.getElementUri(minimbase, "#missingShouldRequirement")
        (g,evalresult) = ro_eval_minim.evaluate(rometa,
            "Minim-UserRequirements.rdf",               # Minim file
            "docs/UserRequirements-bio.html",           # Target resource
            "create")                                   # Purpose
        missing_should = (
            { 'level': "SHOULD"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates docs/missing.css")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("docs/missing.css")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css") 
            })
        self.maxDiff=None
        self.assertEquals(evalresult['summary'],       [MINIM.minimallySatisfies])
        self.assertEquals(evalresult['missingMust'],   [] )
        self.assertEquals(evalresult['missingShould'], [(missing_should,{})] )
        self.assertEquals(evalresult['missingMay'],    [] )
        self.deleteTestRo(rodir)
        return

    def testEvalMayMissing(self):
        """
        Evaluate complete RO against Minim description 
        """
        self.setupConfig()
        rodir      = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        self.populateTestRo(testbase, rodir)
        rometa = ro_metadata(ro_config, rodir)
        minimbase  = rometa.getComponentUri("Minim-UserRequirements.rdf")
        modeluri   = ro_minim.getElementUri(minimbase, "#missingMayRequirement")
        (g,evalresult) = ro_eval_minim.evaluate(rometa,
            "Minim-UserRequirements.rdf",               # Minim file
            "docs/UserRequirements-bio.pdf",            # Target resource
            "create")                                   # Purpose
        missing_may = (
            { 'level': "MAY"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates docs/missing.css")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("docs/missing.css")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css") 
            })
        self.maxDiff=None
        self.assertEquals(evalresult['summary'],        [MINIM.nominallySatisfies, MINIM.minimallySatisfies])
        self.assertEquals(evalresult['missingMust'],    [] )
        self.assertEquals(evalresult['missingShould'],  [] )
        self.assertEquals(evalresult['missingMay'],     [(missing_may,{})] )
        self.assertEquals(evalresult['rouri'],          rometa.getRoUri())
        self.assertEquals(evalresult['minimuri'],       rometa.getComponentUri("Minim-UserRequirements.rdf"))
        self.assertEquals(evalresult['target'],         "docs/UserRequirements-bio.pdf")
        self.assertEquals(evalresult['purpose'],        "create")
        self.assertEquals(evalresult['constrainturi'],  
            rometa.getComponentUri("Minim-UserRequirements.rdf#create/docs/UserRequirements-bio.pdf"))
        self.assertEquals(evalresult['modeluri'],
            rometa.getComponentUri("Minim-UserRequirements.rdf#missingMayRequirement"))
        self.deleteTestRo(rodir)
        self.deleteTestRo(rodir)
        return

    def setupEvalFormat(self):
        self.setupConfig()
        rodir       = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        rometa = ro_metadata(ro_config, rodir)
        minimbase  = rometa.getComponentUri("Minim-UserRequirements.rdf")
        modeluri    = ro_minim.getElementUri(minimbase, "#test-formatting-constraint")
        self.missing_must = (
            { 'level': "MUST"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates data/UserRequirements-bio.ods")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("data/UserRequirements-bio.ods")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-bio.ods")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/data/UserRequirements-bio.ods") 
            })
        self.missing_should = (
            { 'level': "SHOULD"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates docs/missing.css")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("docs/missing.css")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css") 
            })
        self.missing_may = (
            { 'level': "MAY"
            , 'model': modeluri 
            , 'label': rdflib.Literal("aggregates docs/missing.css")
            , 'datarule':
              { 'aggregates': rometa.getComponentUri("docs/missing.css")
              , 'show':       None
              , 'showpass':   None
              , 'showfail':   None
              , 'derives':    ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css")
              }
            , 'uri': ro_minim.getElementUri(minimbase, "#isPresent/docs/missing.css") 
            })
        self.eval_result = (
            { 'summary':        [MINIM.nominallySatisfies, MINIM.minimallySatisfies]
            , 'missingMust':    [(self.missing_must,   {})]
            , 'missingShould':  [(self.missing_should, {})]
            , 'missingMay':     [(self.missing_may,    {})]
            , 'satisfied':      []
            , 'rodir':          rodir
            , 'rouri':          rometa.getRoUri()
            , 'minimuri':       minimbase
            , 'target':         "test-formatting-target"
            , 'purpose':        "test formatting"
            , 'constrainturi':  rometa.getComponentUri("Minim-UserRequirements.rdf#test-formatting-constraint")
            , 'modeluri':       rometa.getComponentUri("Minim-UserRequirements.rdf#test-formatting-constraint")
            })
        self.deleteTestRo(rodir)
        return rodir

    def testEvalFormatSummary(self):
        rodir   = self.setupEvalFormat()
        options = { 'detail': "summary" }
        stream  = StringIO.StringIO()
        ro_eval_minim.format(self.eval_result, options, stream)
        outtxt = stream.getvalue()
        expect = (
            "Research Object file://%s/:\n"%rodir +
            "Nominally complete for %(purpose)s of resource %(target)s\n"%(self.eval_result)
            )
        self.assertEquals(outtxt, expect)
        return

    def testEvalFormatDetail(self):
        rodir   = self.setupEvalFormat()
        options = { 'detail': "full" }
        stream  = StringIO.StringIO()
        ro_eval_minim.format(self.eval_result, options, stream)
        expect = (
            [ "Research Object file://%s/:"%rodir
            , "Nominally complete for %(purpose)s of resource %(target)s"%(self.eval_result)
            , "Unsatisfied MUST requirements:"
            , "  Aggregates resource %s"%(self.eval_result['missingMust'][0][0]['datarule']['aggregates'])
            , "Unsatisfied SHOULD requirements:"
            , "  Aggregates resource %s"%(self.eval_result['missingShould'][0][0]['datarule']['aggregates'])
            , "Unsatisfied MAY requirements:"
            , "  Aggregates resource %s"%(self.eval_result['missingMay'][0][0]['datarule']['aggregates'])
            , "Research object URI:     %(rouri)s"%(self.eval_result)
            , "Minimum information URI: %(minimuri)s"%(self.eval_result)
            ])
        stream.seek(0)
        for expect_line in expect:
            line = stream.readline()
            self.assertEquals(line, expect_line+"\n")
        return

    def testEvaluateChecklistCommand(self):
        self.setupConfig()
        rodir = self.createTestRo(testbase, "data", "RO test minim", "ro-testMinim")
        self.populateTestRo(testbase, rodir)
        rometa   = ro_metadata(ro_config, rodir)
        minimuri = rometa.getComponentUri("Minim-UserRequirements.rdf")
        # Evaluate annotations
        args = [ "ro", "evaluate", "checklist"
               , "-a"
               , "-d", rodir+"/"
               , "Minim-UserRequirements.rdf"
               , "create"
               , "docs/UserRequirements-bio.html"
               ]
        self.outstr.seek(0)
        with StdoutContext.SwitchStdout(self.outstr):
            status = ro.runCommand(
                os.path.join(testbase, TestConfig.ro_test_config.CONFIGDIR),
                os.path.join(testbase, TestConfig.ro_test_config.ROBASEDIR),
                args)
        outtxt = self.outstr.getvalue()
        assert status == 0, "Status %d, outtxt: %s"%(status,outtxt)
        log.debug("status %d, outtxt: %s"%(status, outtxt))
        # Check response returned
        expect = (
            [ "Research Object file://%s/:"%(rodir)
            , "Minimally complete for create of resource docs/UserRequirements-bio.html"
            , "Unsatisfied SHOULD requirements:"
            , "  Aggregates resource %s"%(rometa.getComponentUri("docs/missing.css"))
            , "Satisfied requirements:"
            , "  Aggregates resource %s"%(rometa.getComponentUri("data/UserRequirements-astro.ods")) 
            , "  Environment 'python -V' matches 'Python 2.7'"
            , "Research object URI:     %s"%(rometa.getRoUri())
            , "Minimum information URI: %s"%(minimuri)
            ])
        self.outstr.seek(0)
        for line in self.outstr:
            self.assertIn(str(line)[:-1], expect)
        self.deleteTestRo(rodir)
        return

    # @@TODO Add test cases for software environment rule pass/fail, based on previous

    # @@TODO Add test cases for content match rule pass/fail, based on previous

    # @@TODO Add test cases for forall => exists conteht match
    
    # @@TODO Add test cases for liveness test

    # Test evaluation of remote resource

    def testEvaluateChecklistRemote(self):
        # Config remote testing
        remotehost   = "http://andros.zoo.ox.ac.uk"
        remoterobase = "/workspace/wf4ever-ro-catalogue/v0.1/"
        remoteroname = "simple-requirements/"
        remoteminim  = "simple-requirements-minim.rdf"
        #self.setupConfig()
        rouri    = remotehost+remoterobase+remoteroname
        rometa   = ro_metadata(ro_config, rouri)
        minimuri = rometa.getComponentUri(remoteminim)
        # create rometa object
        rometa = ro_metadata(ro_config, rouri)
        # invoke evaluation service
        #   ro_eval_minim.evaluate(rometa, minim, target, purpose)
        evalresult = ro_eval_minim.evaluate(rometa, minimuri, rouri, "Runnable")
        self.assertEqual(evalresult['rouri'],         rdflib.URIRef(rouri))
        self.assertEqual(evalresult['minimuri'],      rdflib.URIRef(minimuri))
        self.assertEqual(evalresult['target'],        rouri)
        self.assertEqual(evalresult['purpose'],       "Runnable")
        self.assertEqual(evalresult['constrainturi'], rometa.getComponentUri("simple-requirements-minim.rdf#runnable_RO"))
        self.assertEqual(evalresult['modeluri'],      rometa.getComponentUri("simple-requirements-minim.rdf#runnable_RO_model"))
        self.assertIn(MINIM.fullySatisfies,    evalresult['summary'])
        self.assertIn(MINIM.nominallySatisfies, evalresult['summary'])
        self.assertIn(MINIM.minimallySatisfies, evalresult['summary'])
        self.assertEqual(len(evalresult['missingMust']),   0)
        self.assertEqual(len(evalresult['missingShould']), 0)
        self.assertEqual(len(evalresult['missingMay']),    0)
        self.assertEqual(len(evalresult['satisfied']),     4)
        return

    def testEvaluateChecklistRemoteFail(self):
        # Config remote testing
        remotehost   = "http://andros.zoo.ox.ac.uk"
        remoterobase = "/aleix/ro-catalogue/v0.1/"
        remoteroname = "wf74-repeat-fail/"
        remoteminim  = "simple-requirements-minim.rdf"
        #self.setupConfig()
        rouri    = remotehost+remoterobase+remoteroname
        rometa   = ro_metadata(ro_config, rouri)
        minimuri = rometa.getComponentUri(remoteminim)
        # create rometa object
        rometa = ro_metadata(ro_config, rouri)
        # invoke evaluation service
        #   ro_eval_minim.evaluate(rometa, minim, target, purpose)
        evalresult = ro_eval_minim.evaluate(rometa, minimuri, rouri, "Repeatable")
        self.assertEqual(evalresult['rouri'],         rdflib.URIRef(rouri))
        self.assertEqual(evalresult['minimuri'],      rdflib.URIRef(minimuri))
        self.assertEqual(evalresult['target'],        rouri)
        self.assertEqual(evalresult['purpose'],       "Repeatable")
        self.assertEqual(evalresult['constrainturi'], rometa.getComponentUri("simple-requirements-minim.rdf#repeatable_RO"))
        self.assertEqual(evalresult['modeluri'],      rometa.getComponentUri("simple-requirements-minim.rdf#repeatable_RO_model"))
        self.assertNotIn(MINIM.fullySatisfies,     evalresult['summary'])
        self.assertNotIn(MINIM.nominallySatisfies, evalresult['summary'])
        self.assertNotIn(MINIM.minimallySatisfies, evalresult['summary'])
        self.assertEqual(len(evalresult['missingMust']),   1)
        self.assertEqual(len(evalresult['missingShould']), 0)
        self.assertEqual(len(evalresult['missingMay']),    0)
        self.assertEqual(len(evalresult['satisfied']),     4)
        return

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testNull"
            , "testSetupConfig"
            , "testEvalAllPresent"
            , "testEvalMustMissing"
            , "testEvalShouldMissing"
            , "testEvalMayMissing"
            , "testEvalFormatSummary"
            , "testEvalFormatDetail"
            , "testEvaluateChecklistCommand"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            , "testEvaluateChecklistRemote"
            ]
        }
    return TestUtils.getTestSuite(TestEvalChecklist, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestEvalChecklist.log", getTestSuite, sys.argv)

# End.
